# Copyright (c) Alibaba, Inc. and its affiliates.

import json
from dataclasses import dataclass

from rouge import Rouge
import os


@dataclass
class EvalArgs:
    input_path: str
    output_path: str


def run_eval(args: EvalArgs):
    print(f'*** Start evaluation with eval args: {args}\n')

    args.input_path = os.path.join(args.input_path, 'predictions.json')
    args.output_path = os.path.join(args.output_path, 'metrics.json')

    def evaluate_rougel(cand_list: list, ref_list: list):
        if len(ref_list) == 0:
            return 0
        rouge = Rouge()
        rouge_score = rouge.get_scores(hyps=cand_list, refs=ref_list, avg=True)
        rougel = rouge_score["rouge-l"]["f"]
        return rougel

    def evaluate_action_em(cand_list: list, ref_list: list):
        if len(ref_list) == 0:
            return 0
        em = 0
        for cand, ref in zip(cand_list, ref_list):
            em += (1 if cand == ref else 0)
        return em / len(cand_list)

    def evaluate_action_input_f1(action_pred: list, action_ref: list, cand_list: list, ref_list: list):
        easy_f1 = []
        hard_f1 = []
        f1 = []
        for i in range(len(action_pred)):
            ref_action = action_ref[i]
            pred_action = action_pred[i]

            ref_input = ref_list[i]
            cand_input = cand_list[i]

            if ref_action != pred_action:
                easy_f1.append(0)
                hard_f1.append(0)
                f1.append(0)
            else:
                try:
                    ref_input_json = json.loads(ref_input)
                    try:
                        cand_input_json = json.loads(cand_input)
                        half_match = 0
                        full_match = 0
                        if ref_input_json == {}:
                            if cand_input_json == {}:
                                easy_f1.append(1)
                                f1.append(1)
                            else:
                                easy_f1.append(0)
                                f1.append(0)
                        else:
                            for k, v in ref_input_json.items():
                                if k in cand_input_json.keys():
                                    if cand_input_json[k] == v:
                                        full_match += 1
                                    else:
                                        half_match += 1

                            recall = (0.5 * half_match + full_match) / (len(ref_input_json) + 1e-30)
                            precision = (0.5 * half_match + full_match) / (len(cand_input_json) + 1e-30)
                            hard_f1.append((2 * recall * precision) / (recall + precision))
                            f1.append((2 * recall * precision) / (recall + precision))
                    except:
                        # cand_input = cand_input.replace("\n","").replace("\"","")
                        # ref_input = cand_input.replace("\n","").replace("\"","")
                        # rouge = Rouge()
                        # rouge_score = rouge.get_scores(hyps=[cand_input], refs=[ref_input], avg=True)
                        if ref_input_json == {}:
                            easy_f1.append(0)
                        else:
                            hard_f1.append(0)
                        # hard_f1.append(rouge_score["rouge-l"]["f"])
                        # f1.append(rouge_score["rouge-l"]["f"])
                        f1.append(0)
                except:
                    pass

        return sum(easy_f1) / len(easy_f1) + 1e-30, sum(hard_f1) / len(hard_f1) + 1e-30, sum(f1) / len(f1) + 1e-30

    with open(args.input_path, encoding='utf-8') as f:
        data = json.load(f)

    def parse_action(text):
        action = "None"
        action_input = "{}"
        if 'Action Input:' in text:
            input_idx = text.rindex('Action Input:')
            action_input = text[input_idx + len('Action Input:'):].strip()
        else:
            action_input = '{}'

        if 'Action:' in text:
            action_idx = text.rindex('Action:')
            action = text[action_idx + len('Action:'):].strip()
            if 'Action Input:' in action:
                input_idx = action.index('Action Input:')
                action = action[:input_idx].strip()
        else:
            action = 'none'
        return action, action_input

    def parse_output(text):
        action, action_input = parse_action(text)
        if action == "Finish":
            try:
                action_input = json.loads(action_input)
                # print(action_input)
                # print(json.dumps(action_input,indent=2))
                return_type = action_input["return_type"]
                if return_type == "give_answer":
                    if "final_answer" in action_input.keys():
                        answer = str(action_input['final_answer'])
                        if answer.strip() in ['', '.', ',']:
                            answer = "None"
                    else:
                        answer = "None"
                    return "finish", action, action_input, answer
                else:
                    return "give up", None, None, None
            except:
                return "give up", None, None, None
        else:
            plan = 'call'
            answer = None
            return plan, action, action_input, answer

    plan_ref = []
    plan_pred = []
    hallu_cases = []
    error_cases = []
    new_data = []
    answer_ref = []
    action_ref = []
    action_input_ref = []
    hallu_ref = 0
    answer_pred = []
    action_pred = []
    action_input_pred = []
    hallu_pred = 0
    for d in data:
        reference = d['target']
        prediction = d['predictions']
        ref_plan, ref_action, ref_input, ref_ans = parse_output(reference)
        # ref_plan: call
        # ref_action: spott
        # ref_input: {"is_id": "city center" }
        # ref_ans: None

        pred_plan, pred_action, pred_input, pred_ans = parse_output(prediction)
        if ref_action is not None and ref_action == "invalid_hallucination_function_name":
            continue
        if pred_action is not None and ref_action != 'none' and ref_action not in [t['name'] for t in d['tools']]:
            continue

        if pred_action is not None and pred_action != 'none' and pred_action not in [t['name'] for t in d['tools']]:
            hallu_pred += 1
            hallu_cases.append(d)

        plan_ref.append(ref_plan)
        plan_pred.append(pred_plan)
        if ref_plan == 'give up':
            pass
        elif ref_plan == 'finish':
            answer_ref.append(ref_ans)
            if pred_ans is None:
                answer_pred.append('none')
            else:
                answer_pred.append(pred_ans)
        else:
            action_ref.append(ref_action)
            action_input_ref.append(ref_input)
            if pred_action is None:
                action_pred.append('none')
            else:
                action_pred.append(pred_action)

            if pred_input is None:
                action_input_pred.append('{}')
            else:
                action_input_pred.append(pred_input)

    metric = {}
    rouge = evaluate_rougel(answer_pred, answer_ref)
    plan_em = evaluate_action_em(cand_list=plan_pred, ref_list=plan_ref)
    action_em = evaluate_action_em(cand_list=action_pred, ref_list=action_ref)
    easy_f1, hard_f1, f1 = evaluate_action_input_f1(action_pred, action_ref, action_input_pred, action_input_ref)
    hallu_rate = hallu_pred / len(data)
    metric['rouge'] = rouge
    metric['plan_em'] = plan_em
    metric['action_em'] = action_em
    metric['easy_f1'] = easy_f1
    metric['hard_f1'] = hard_f1
    metric['f1'] = f1
    metric['hallu_rate'] = hallu_rate

    if not os.path.exists(os.path.dirname(args.output_path)):
        os.makedirs(os.path.dirname(args.output_path))
    print(metric)
    with open(args.output_path, 'w', encoding='utf-8') as f:
        json.dump(metric, f, indent=2)

    with open(args.output_path.replace('metrics.json', 'hallu_cases.json'), 'w', encoding='utf-8') as f:
        json.dump(hallu_cases, f, indent=2)



import os
import numpy as np
from vlmeval.dataset.image_base import ImageBaseDataset
from vlmeval.dataset.image_vqa import CustomVQADataset
from vlmeval.smp import load, dump, d2df

class CustomDataset:

    def load_data(self, dataset):
        # customize the loading of the dataset
        data_path = os.path.join(os.path.expanduser("~/LMUData"), f'{dataset}.tsv')
        return load(data_path)


    def build_prompt(self, line):
        msgs = ImageBaseDataset.build_prompt(self, line)
        # add a hint or custom instruction here
        msgs[-1]['value'] += '\nAnswer the question using a single word or phrase.'
        return msgs
    
    
    def evaluate(self, eval_file, **judge_kwargs):
        data = load(eval_file)
        assert 'answer' in data and 'prediction' in data
        data['prediction'] = [str(x) for x in data['prediction']]
        data['answer'] = [str(x).lower() for x in data['answer']]
        
        print(data)
        
        # ========compute the evaluation metrics as you need =========
        # exact match
        result = np.mean(data['answer'] == data['prediction'])
        ret = {'Overall': result}
        ret = d2df(ret).round(2)

        # save the result
        suffix = eval_file.split('.')[-1]
        result_file = eval_file.replace(f'.{suffix}', '_acc.csv')
        dump(ret, result_file)
        return ret
        # ============================================================


# override the default dataset class
CustomVQADataset.load_data = CustomDataset.load_data
CustomVQADataset.build_prompt = CustomDataset.build_prompt
CustomVQADataset.evaluate = CustomDataset.evaluate



from dataclasses import dataclass, field
from typing import List, Optional, Union, Dict, Any


@dataclass
class TestsetGenerationArguments:
    docs: List[str] = field(default_factory=list)
    test_size: int = 10
    output_file: str = 'outputs/testset.json'
    knowledge_graph: str = 'outputs/knowledge_graph.json'
    """
    For local LLM support, you can use the following fields:
        model_name_or_path: str
        model_revision: str = "master"
        template_type: str = "default"
        generation_config: Optional[Dict]

    For API LLM support, you can use the following fields:
        model_name="gpt-4o-mini"
        api_base: str = "",
        api_key: Optional[str] = None
    """
    generator_llm: Dict = field(default_factory=dict)
    embeddings: Dict = field(default_factory=dict)
    distribution: str = field(
        default_factory=lambda: {'simple': 0.5, 'multi_context': 0.4, 'reasoning': 0.1}
    )
    # For LLM based evaluation
    # available: ['english', 'hindi', 'marathi', 'chinese', 'spanish', 'amharic', 'arabic',
    # 'armenian', 'bulgarian', 'urdu', 'russian', 'polish', 'persian', 'dutch', 'danish',
    # 'french', 'burmese', 'greek', 'italian', 'japanese', 'deutsch', 'kazakh', 'slovak']
    language: str = 'english'


@dataclass
class EvaluationArguments:
    testset_file: str
    critic_llm: Dict = field(default_factory=dict)
    embeddings: Dict = field(default_factory=dict)
    metrics: List[str] = field(
        default_factory=lambda: ['answer_relevancy', 'faithfulness']
    )
    # For LLM based evaluation
    # available: ['english', 'hindi', 'marathi', 'chinese', 'spanish', 'amharic', 'arabic',
    # 'armenian', 'bulgarian', 'urdu', 'russian', 'polish', 'persian', 'dutch', 'danish',
    # 'french', 'burmese', 'greek', 'italian', 'japanese', 'deutsch', 'kazakh', 'slovak']
    language: str = 'english'



# Copyright (c) Alibaba, Inc. and its affiliates.
import os
import unittest
from evalscope.utils import test_level_list, is_module_installed
from evalscope.utils.logger import get_logger
from evalscope.run import run_task

logger = get_logger()


class TestRAGAS(unittest.TestCase):

    def setUp(self) -> None:
        self._check_env('ragas')

    def tearDown(self) -> None:
        pass

    @staticmethod
    def _check_env(module_name: str):
        if is_module_installed(module_name):
            logger.info(f'{module_name} is installed.')
        else:
            raise ModuleNotFoundError(f'run: pip install {module_name}')

    @unittest.skipUnless(0 in test_level_list(), 'skip test in current test level')
    def test_run_generate_dataset(self):
        task_cfg = {
            'eval_backend': 'RAGEval',
            'eval_config': {
                'tool': 'RAGAS',
                'testset_generation': {
                    'docs': ['README_zh.md'],
                    'test_size': 5,
                    'output_file': 'outputs/testset.json',
                    'distribution': {
                        'simple': 0.5,
                        'multi_context': 0.4,
                        'reasoning': 0.1,
                    },
                    'generator_llm': {
                        'model_name_or_path': 'qwen/Qwen2-7B-Instruct',
                        'template_type': 'qwen',
                    },
                    'embeddings': {
                        'model_name_or_path': 'AI-ModelScope/m3e-base',
                    },
                    'language': 'chinese',
                },
            },
        }

        logger.info(f'>> Start to run task: {task_cfg}')

        run_task(task_cfg)

    @unittest.skipUnless(0 in test_level_list(), 'skip test in current test level')
    def test_run_rag_eval(self):
        task_cfg = {
            'eval_backend': 'RAGEval',
            'eval_config': {
                'tool': 'RAGAS',
                'eval': {
                    'testset_file': 'outputs/testset_chinese_with_answer.json',
                    'critic_llm': {
                        'model_name_or_path': 'qwen/Qwen2-7B-Instruct',
                        'template_type': 'qwen',
                    },
                    'embeddings': {
                        'model_name_or_path': 'AI-ModelScope/m3e-base',
                    },
                    'metrics': [
                        'Faithfulness',
                        'AnswerRelevancy',
                        'ContextPrecision',
                        'AnswerCorrectness',
                    ],
                },
            },
        }

        logger.info(f'>> Start to run task: {task_cfg}')

        run_task(task_cfg)

    @unittest.skipUnless(0 in test_level_list(), 'skip test in current test level')
    def test_run_rag_eval_api(self):
        task_cfg = {
            'eval_backend': 'RAGEval',
            'eval_config': {
                'tool': 'RAGAS',
                'eval': {
                    'testset_file': 'outputs/testset.json',
                    'critic_llm': {
                        'model_name': 'gpt-4o-mini',  # 自定义聊天模型名称
                        'api_base': 'http://127.0.0.1:8088/v1',  # 自定义基础URL
                        'api_key': 'xxxx',  # 你的API密钥
                    },
                    'embeddings': {
                        'model_name_or_path': 'AI-ModelScope/m3e-base',
                    },
                    'metrics': [
                        'Faithfulness',
                        'AnswerRelevancy',
                        'ContextPrecision',
                        'AnswerCorrectness',
                        'MultiModalFaithfulness',
                        'MultiModalRelevance',
                    ],
                },
            },
        }

        logger.info(f'>> Start to run task: {task_cfg}')

        run_task(task_cfg)


if __name__ == '__main__':
    unittest.main(buffer=False)



import os
import torch
from typing import List, Optional, Union, Dict
from sentence_transformers import models
from sentence_transformers.SentenceTransformer import SentenceTransformer
from sentence_transformers.cross_encoder import CrossEncoder
from torch import Tensor
from evalscope.backend.rag_eval.utils.tools import download_model
from evalscope.utils.logger import get_logger
from langchain_core.embeddings import Embeddings

logger = get_logger()


class BaseModel(Embeddings):
    def __init__(
        self,
        model_name_or_path: str,
        max_seq_length: int = 512,
        prompt: str = '',
        revision: Optional[str] = None,
        **kwargs,
    ):
        self.model_name_or_path = model_name_or_path
        self.max_seq_length = max_seq_length
        self.model_kwargs = kwargs.pop('model_kwargs', {})
        self.model_kwargs['trust_remote_code'] = True

        self.config_kwargs = kwargs.pop('config_kwargs', {})
        self.config_kwargs['trust_remote_code'] = True

        self.encode_kwargs = kwargs.pop('encode_kwargs', {})
        self.encode_kwargs['convert_to_tensor'] = True

        self.prompt = prompt
        self.revision = revision

    @property
    def mteb_model_meta(self):
        """Model metadata for MTEB (Multilingual Task Embeddings Benchmark)"""
        from mteb import ModelMeta

        return ModelMeta(
            name=os.path.basename(self.model_name_or_path),
            revision=self.revision,
            languages=None,
            release_date=None,
        )

    def embed_documents(self, texts: List[str]) -> List[List[float]]:
        """Embed search docs. Compact langchain.

        Args:
            texts: List of text to embed.

        Returns:
            List of embeddings.
        """
        return self.encode_corpus(texts).tolist()

    def embed_query(self, text: str) -> List[float]:
        """Embed query text. Compact langchain.

        Args:
            text: Text to embed.

        Returns:
            Embedding.
        """
        return self.encode_queries(text).tolist()

    def encode(self, texts: Union[str, List[str]], **kwargs) -> List[List[float]]:
        """Embed text."""
        raise NotImplementedError

    def encode_queries(self, queries: List[str], **kwargs) -> list[torch.Tensor]:
        """Embed query text. Compact mteb."""
        raise NotImplementedError

    def encode_corpus(self, corpus: List[str] | List[Dict[str, str]], **kwargs) -> list[torch.Tensor]:
        """Embed search docs . Compact mteb."""
        raise NotImplementedError


class SentenceTransformerModel(BaseModel):
    def __init__(
        self, model_name_or_path: str, pooling_mode: Optional[str] = None, **kwargs
    ):
        super().__init__(model_name_or_path, **kwargs)

        if not pooling_mode:
            self.model = SentenceTransformer(
                self.model_name_or_path,
                config_kwargs=self.config_kwargs,
                model_kwargs=self.model_kwargs,
            )
        else:
            word_embedding_model = models.Transformer(
                self.model_name_or_path,
                config_args=self.config_kwargs,
                model_args=self.model_kwargs,
            )
            pooling_model = models.Pooling(
                word_embedding_model.get_word_embedding_dimension(),
                pooling_mode=pooling_mode,
            )
            self.model = SentenceTransformer(
                modules=[word_embedding_model, pooling_model],
            )

        self.model.max_seq_length = self.max_seq_length

    def encode(self, texts: Union[str, List[str]], prompt=None, **kwargs) -> List[torch.Tensor]:
        kwargs.pop('prompt_name', '')  # remove prompt name, use prompt
        self.encode_kwargs.update(kwargs)

        embeddings = self.model.encode(texts, prompt=prompt, **self.encode_kwargs)
        assert isinstance(embeddings, Tensor)
        return embeddings.cpu().detach()

    def encode_queries(self, queries, **kwargs):
        return self.encode(queries, prompt=self.prompt)

    def encode_corpus(self, corpus, **kwargs):
        if isinstance(corpus[0], dict):
            input_texts = ['{} {}'.format(doc.get('title', ''), doc['text']).strip() for doc in corpus]
        else:
            input_texts = corpus
        return self.encode(input_texts)


class CrossEncoderModel(BaseModel):
    def __init__(self, model_name_or_path: str, **kwargs):
        super().__init__(model_name_or_path, **kwargs)
        self.model = CrossEncoder(
            self.model_name_or_path,
            trust_remote_code=True,
            max_length=self.max_seq_length,
        )

    def predict(self, sentences: List[List[str]], **kwargs) -> List[List[float]]:
        self.encode_kwargs.update(kwargs)

        if len(sentences[0]) == 3:  # Note: For mteb retrieval task
            processed_sentences = []
            for query, docs, instruction in sentences:
                if isinstance(docs, dict):
                    docs = docs['text']
                processed_sentences.append((self.prompt + query, docs))
            sentences = processed_sentences
        embeddings = self.model.predict(sentences, **self.encode_kwargs)
        assert isinstance(embeddings, Tensor)
        return embeddings


class EmbeddingModel:
    """Custom embeddings"""

    @staticmethod
    def load(
        model_name_or_path: str = '',
        is_cross_encoder: bool = False,
        hub: str = 'modelscope',
        revision: Optional[str] = 'master',
        **kwargs,
    ):
        # If model path does not exist and hub is 'modelscope', download the model
        if not os.path.exists(model_name_or_path) and hub == 'modelscope':
            model_name_or_path = download_model(model_name_or_path, revision)

        # Return different model instances based on whether it is a cross-encoder and pooling mode
        if is_cross_encoder:
            return CrossEncoderModel(
                model_name_or_path,
                revision=revision,
                **kwargs,
            )
        else:
            return SentenceTransformerModel(
                model_name_or_path,
                revision=revision,
                **kwargs,
            )



# Copyright (c) Alibaba, Inc. and its affiliates.

import os

# os.environ["CUDA_VISIBLE_DEVICES"] = "1"
import subprocess
import unittest
from evalscope.utils import test_level_list, is_module_installed
from evalscope.utils.logger import get_logger
from evalscope.run import run_task

logger = get_logger()


class TestCLIPBenchmark(unittest.TestCase):

    def setUp(self) -> None:
        self._check_env("webdataset")

    def tearDown(self) -> None:
        pass

    @staticmethod
    def _check_env(module_name: str):
        if is_module_installed(module_name):
            logger.info(f"{module_name} is installed.")
        else:
            raise ModuleNotFoundError(f"run: pip install {module_name}")

    @unittest.skipUnless(0 in test_level_list(), "skip test in current test level")
    def test_run_task(self):
        task_cfg = {
            "eval_backend": "RAGEval",
            "eval_config": {
                "tool": "clip_benchmark",
                "eval": {
                    "models": [
                        {
                            "model_name": "AI-ModelScope/chinese-clip-vit-large-patch14-336px",
                        }
                    ],
                    "dataset_name": ["muge", "mnist"],
                    "split": "test",
                    "batch_size": 128,
                    "num_workers": 1,
                    "verbose": True,
                    "skip_existing": False,
                    "output_dir": "outputs",
                    "cache_dir": "cache",
                    "limit": 1000,
                },
            },
        }

        run_task(task_cfg)

    @unittest.skipUnless(0 in test_level_list(), "skip test in current test level")
    def test_run_custom(self):
        task_cfg = {
            "eval_backend": "RAGEval",
            "eval_config": {
                "tool": "clip_benchmark",
                "eval": {
                    "models": [
                        {
                            "model_name": "AI-ModelScope/chinese-clip-vit-large-patch14-336px",
                        }
                    ],
                    "dataset_name": ["custom"],
                    "data_dir": "custom_eval/multimodal/text-image-retrieval",
                    "split": "test",
                    "batch_size": 128,
                    "num_workers": 1,
                    "verbose": True,
                    "skip_existing": False,
                    "limit": 1000,
                },
            },
        }

        run_task(task_cfg)


if __name__ == "__main__":
    unittest.main(buffer=False)



# Convert datasets to webdataset format
import os
from tqdm import tqdm
import torch
import torch.utils.data
import webdataset
from evalscope.backend.rag_eval.clip_benchmark.dataset_builder import DatasetWrapper
from evalscope.backend.rag_eval.utils.tools import path_to_bytes, PIL_to_bytes
from evalscope.utils.logger import get_logger

logger = get_logger()


def convert_dataset(
    dataset,
    split,
    output_folder,
    *,
    transform=None,
    image_format="webp",
    max_count=10_000,
    max_size=1_000_000_000,
    multilabel=False,
    verbose=True,
):
    """
    Convert an iterable `dataset` of (image, label) pairs to webdataset (.tar) format, and store in `output_folder/split`.

    Images may be passed in as either:
    * File paths: pass in `transform=path_to_bytes`;
    * PIL images: pass in `transform=PIL_to_bytes(image_format)` where `image_format` is e.g. "webp"; or
    * Raw binary data: use a PyTorch `Dataset` that supports `transform=PIL_to_bytes(image_format)`, and pass in `transform=None` here.
        Be sure that the transform is not applied twice.

    Copying image files directly or writing raw binary data is fastest since it allows multiprocessing;
    passing in PIL images will be slower, but should work for any format of dataset.

    Labels must be zero-indexed integers (for multilabel datasets, labels must be arrays/tensors).

    Classnames and zero-shot classification templates can be provided as attributes of the dataset (`.classes` and `.templates`)
    or filled in manually afterward. `dataset.classes` should be a list of strings indexed by the labels,
    and `dataset.templates` should be a list of strings containing `{c}` to specify where classnames are to be inserted.
    """
    # Create output directory
    os.makedirs(os.path.join(output_folder, split), exist_ok=True)
    # Multiprocessed dataloader, should work with Dataset or list
    dataloader = torch.utils.data.DataLoader(
        dataset,
        batch_size=1,
        num_workers=8,
        collate_fn=lambda batch: batch[0],  # No collate, only for multiprocessing
    )
    if verbose:
        try:
            logger.info(f"Dataset size: {len(dataset)}")
        except TypeError:
            logger.info("IterableDataset has no len()")
    # Save classnames
    if hasattr(dataset, "classes") and dataset.classes:
        classnames_fname = os.path.join(output_folder, "classnames.txt")
        with open(classnames_fname, "w") as classnames_file:
            logger.info(*dataset.classes, sep="\n", end="\n", file=classnames_file)
        if verbose:
            logger.info("Saved class names to '%s'" % classnames_fname)
    elif verbose:
        logger.info("WARNING: No class names found")
    # Save zeroshot templates
    if hasattr(dataset, "templates") and dataset.templates:
        templates_fname = os.path.join(
            output_folder, "zeroshot_classification_templates.txt"
        )
        with open(templates_fname, "w") as templates_file:
            logger.info(*dataset.templates, sep="\n", end="\n", file=templates_file)
        if verbose:
            logger.info("Saved class names to '%s'" % templates_fname)
    elif verbose:
        logger.info("WARNING: No zeroshot classification templates found")
    # Save dataset type
    if multilabel:
        type_fname = os.path.join(output_folder, "dataset_type.txt")
        with open(type_fname, "w") as type_file:
            logger.info("multilabel", end="\n", file=type_file)
            if verbose:
                logger.info("Saved dataset type to '%s'" % type_fname)
    # Write to TAR files
    data_fname = os.path.join(output_folder, split, r"%d.tar")
    sink = webdataset.ShardWriter(data_fname, maxcount=max_count, maxsize=max_size)
    nsamples = 0
    label_type = "npy" if multilabel else "cls"
    for index, (input, output) in enumerate(tqdm(dataloader, desc="Converting")):
        nsamples += 1
        if isinstance(input, str) and transform is path_to_bytes:
            # If copying file, determine image format from extension
            extension = (
                os.path.splitext(input)[1]
                .replace(".", "")
                .lower()
                .replace("jpeg", "jpg")
                or image_format
            )
        else:
            extension = image_format
        # Convert label if necessary
        if isinstance(output, torch.Tensor):
            if multilabel:
                output = output.detach().cpu().numpy()
            else:
                output = output.item()
        # Write example
        sink.write(
            {
                "__key__": "s%07d" % index,
                extension: transform(input) if transform else input,
                label_type: output,
            }
        )
    num_shards = sink.shard
    sink.close()
    if verbose:
        logger.info(
            "Saved dataset to '%s'"
            % data_fname.replace(r"%d", "{0..%d}" % (num_shards - 1))
        )
    # Save number of shards
    nshards_fname = os.path.join(output_folder, split, "nshards.txt")
    with open(nshards_fname, "w") as nshards_file:
        logger.info(num_shards, end="\n", file=nshards_file)
    if verbose:
        logger.info("Saved number of shards = %d to '%s'" % (num_shards, nshards_fname))
    logger.info("Final dataset size:", nsamples)


def convert_retrieval_dataset(
    dataset,
    split,
    output_folder,
    *,
    transform=None,
    image_format="webp",
    max_count=10_000,
    max_size=1_000_000_000,
    verbose=True,
):
    """
    Convert an iterable `dataset` of (image, [caption1, caption2, ...]) pairs to webdataset (.tar) format, and store in `output_folder/split`.

    Labels must be lists of strings, with no newlines.

    Read the documentation of `convert_dataset` for more information.
    """
    # Create output directory
    os.makedirs(os.path.join(output_folder, split), exist_ok=True)
    # Multiprocessed dataloader, should work with Dataset or list
    dataloader = torch.utils.data.DataLoader(
        dataset,
        batch_size=1,
        num_workers=8,
        collate_fn=lambda batch: batch[0],  # No collate, only for multiprocessing
    )
    if verbose:
        try:
            logger.info(f"Dataset size: {len(dataset)}")
        except TypeError:
            logger.info("IterableDataset has no len()")
    # No classnames
    # No zeroshot templates
    # Save dataset type
    type_fname = os.path.join(output_folder, "dataset_type.txt")
    with open(type_fname, "w") as type_file:
        logger.info("retrieval", end="\n", file=type_file)
    if verbose:
        logger.info("Saved dataset type to '%s'" % type_fname)
    # Write to TAR files
    data_fname = os.path.join(output_folder, split, r"%d.tar")
    sink = webdataset.ShardWriter(data_fname, maxcount=max_count, maxsize=max_size)
    nsamples = 0
    for index, (input, output) in enumerate(tqdm(dataloader, desc="Converting")):
        nsamples += 1
        if isinstance(input, str) and transform is path_to_bytes:
            # If copying file, determine image format from extension
            extension = (
                os.path.splitext(input)[1]
                .replace(".", "")
                .lower()
                .replace("jpeg", "jpg")
                or image_format
            )
        else:
            extension = image_format
        sink.write(
            {
                "__key__": "s%07d" % index,
                extension: transform(input) if transform else input,
                "txt": "\n".join(caption.replace("\n", r"\n") for caption in output),
            }
        )
    num_shards = sink.shard
    sink.close()
    if verbose:
        logger.info(
            "Saved dataset to '%s'"
            % data_fname.replace(r"%d", "{0..%d}" % (num_shards - 1))
        )
    # Save number of shards
    nshards_fname = os.path.join(output_folder, split, "nshards.txt")
    with open(nshards_fname, "w") as nshards_file:
        logger.info(num_shards, end="\n", file=nshards_file)
    if verbose:
        logger.info("Saved number of shards = %d to '%s'" % (num_shards, nshards_fname))
    logger.info("Final dataset size:", nsamples)


if __name__ == "__main__":
    from modelscope.msdatasets import MsDataset

    splits = ["train", "validation"]
    for split in splits:
        ds = MsDataset.load("modelscope/muge", split=split)
        hf_dataset = ds.to_hf_dataset()
        pytorch_dataset = DatasetWrapper(
            hf_dataset, image_key="image", text_key="query"
        )
        convert_retrieval_dataset(
            pytorch_dataset,
            split,
            "data/muge",
            transform=PIL_to_bytes("jpg"),
            image_format="jpg",
            max_count=50_000,
        )



from dataclasses import dataclass, field
from typing import List, Optional, Union, Dict, Any


@dataclass
class ModelArguments:
    # Arguments for embeding model: sentence transformer or cross encoder
    model_name_or_path: str = ""  # model name or path
    is_cross_encoder: bool = False  # whether the model is a cross encoder
    # pooling mode: Either “cls”, “lasttoken”, “max”, “mean”, “mean_sqrt_len_tokens”, or “weightedmean”.
    pooling_mode: Optional[str] = None
    max_seq_length: int = 512  # max sequence length
    # prompt for llm based model
    prompt: str = ""
    # model kwargs
    model_kwargs: dict = field(default_factory=dict)
    # config kwargs
    config_kwargs: Dict[str, Any] = field(default_factory=dict)
    # encode kwargs
    encode_kwargs: dict = field(
        default_factory=lambda: {"show_progress_bar": True, "batch_size": 32}
    )
    hub: str = "modelscope"  # modelscope or huggingface

    def to_dict(self) -> Dict[str, Any]:
        return {
            "model_name_or_path": self.model_name_or_path,
            "is_cross_encoder": self.is_cross_encoder,
            "pooling_mode": self.pooling_mode,
            "max_seq_length": self.max_seq_length,
            "prompt": self.prompt,
            "model_kwargs": self.model_kwargs,
            "config_kwargs": self.config_kwargs,
            "encode_kwargs": self.encode_kwargs,
            "hub": self.hub,
        }


@dataclass
class EvalArguments:
    # Evaluation
    tasks: List[str] = field(default_factory=list)  # task names
    dataset_path: Optional[str] = None  # custom dataset path
    verbosity: int = 2  # verbosity level 0-3
    output_folder: str = "outputs"  # output folder
    overwrite_results: bool = True  # overwrite results
    limits: Optional[int] = None  # limit number of samples
    hub: str = "modelscope"  # modelscope or huggingface
    top_k: int = 5 # top k for reranking

    def to_dict(self) -> Dict[str, Any]:
        return {
            "tasks": self.tasks,
            "dataset_path": self.dataset_path,
            "verbosity": self.verbosity,
            "output_folder": self.output_folder,
            "overwrite_results": self.overwrite_results,
            "limits": self.limits,
            "hub": self.hub,
            "top_k": self.top_k,
        }



import subprocess
from typing import Any, Dict, Iterator, List

import json

from evalscope.perf.arguments import Arguments
from evalscope.perf.plugin.datasets.base import DatasetPluginBase
from evalscope.perf.plugin.registry import register_dataset


@register_dataset('openqa')
class OpenqaDatasetPlugin(DatasetPluginBase):
    """Read dataset and return prompt.
    Datasets: https://www.modelscope.cn/datasets/AI-ModelScope/HC3-Chinese/resolve/master/open_qa.jsonl
    """

    def __init__(self, query_parameters: Arguments):
        super().__init__(query_parameters)

    def build_messages(self) -> Iterator[List[Dict]]:
        if not self.query_parameters.dataset_path:
            subprocess.call([
                'modelscope',
                'download',
                '--dataset',
                'AI-ModelScope/HC3-Chinese',
                'open_qa.jsonl',
                '--local_dir',
                './data',
            ])
            self.query_parameters.dataset_path = './data/open_qa.jsonl'

        for item in self.dataset_line_by_line(self.query_parameters.dataset_path):
            item = json.loads(item)
            prompt = item['question'].strip()
            if (len(prompt) > self.query_parameters.min_prompt_length
                    and len(prompt) < self.query_parameters.max_prompt_length):
                yield [{'role': 'user', 'content': prompt}]



from typing import Any, List, Type


class PluginRegistry:

    def __init__(self):
        self._registry = {}

    def register(self, name, cls):
        self._registry[name] = cls
        return cls

    def get_class(self, name):
        return self._registry[name]

    def all_classes(self):
        return list(self._registry.keys())

    def __call__(self, name: str) -> Any:
        return self.get_class(name)


def register_dataset(name: str | List[str]):

    def class_decorator(cls: Type):
        if isinstance(name, str):
            DatasetRegistry.register(name, cls)
        elif isinstance(name, list):
            for n in name:
                DatasetRegistry.register(n, cls)
        else:
            raise TypeError('name must be a string or a list of strings')
        return cls

    return class_decorator


def register_api(name: str | List[str]):

    def class_decorator(cls: Type):
        if isinstance(name, str):
            ApiRegistry.register(name, cls)
        elif isinstance(name, list):
            for n in name:
                ApiRegistry.register(n, cls)
        else:
            raise TypeError('name must be a string or a list of strings')
        return cls

    return class_decorator


DatasetRegistry = PluginRegistry()
ApiRegistry = PluginRegistry()



# Copyright (c) Alibaba, Inc. and its affiliates.
from enum import Enum
from typing import Optional, Union
import subprocess
from dataclasses import asdict
import tempfile

from evalscope.utils import is_module_installed, get_module_path, get_valid_list
from evalscope.backend.base import BackendManager
from evalscope.backend.opencompass.api_meta_template import get_template
from evalscope.utils.logger import get_logger

logger = get_logger()


class CmdMode(Enum):

    # The basic mode is to run the command directly,
    # e.g. `python -m run --models model1 model2 --datasets dataset1 dataset2`
    BASIC = 'basic'

    # The script mode is to run the command with a script,
    # e.g. `python -m run your_config_script.py`
    SCRIPT = 'script'


class RunMode(Enum):

    # The command mode is to run the command directly with the command line.
    CMD = 'cmd'

    # The function mode is to run the command with a function call -- run_task().
    FUNCTION = 'function'


class OpenCompassBackendManager(BackendManager):

    def __init__(self, config: Union[str, dict], **kwargs):
        """
        The backend manager for OpenCompass.

        Args:
            config: Union[str, dict], the configuration yaml-file or the configuration dictionary.
                attributes:
                    datasets: list, the datasets.
                    models: list, the models.
                    work_dir (Optional): str, the working directory. Default to None, which means the current directory.
                    dry_run (Optional): bool, the dry-run flag. Default to False.
                    debug (Optional): bool, the debug flag. Default to False.
                    reuse (Optional): str, reuse previous outputs & results. Default to None.
                    generation_kwargs (Optional): dict, the generation config. Default to {}.
                    limit (Optional): int or float or str, the limit of the number of examples. Default to None.
                        if limit is a string, it should be in the format of '[start:end]'.

                example:
                    # TODO: add demo config
                    config = dict(
                        datasets=[mmlu, ceval],
                        models=[...],
                        ...
                    )

            **kwargs: the keyword arguments.
        """

        self._check_env()
        super().__init__(config, **kwargs)

        from opencompass.cli.arguments import Arguments as OpenCompassArguments
        self.args = OpenCompassArguments(**self.config_d)

    @property
    def cmd(self):
        return self.get_cmd()

    @staticmethod
    def _check_env():
        if is_module_installed('opencompass'):
            logger.info('Check the OpenCompass environment: OK')

    @staticmethod
    def get_restore_arg(arg_name: str, arg_val: bool):
        if arg_val:
            return f'--{arg_name}'
        else:
            return ''

    @staticmethod
    def get_arg_with_default(arg_name: str, arg_val: Optional[str] = None):
        if arg_val:
            return f'--{arg_name} {arg_val}'
        else:
            return ''

    def load_task_template(self):
        """
        Load the initial OpenCompass task template from task config file.

        Returns:
            (mmengine.config.config.Config), the initial task template config.
        """
        from opencompass.utils.run import get_config_from_arg

        template_config_path = get_module_path('evalscope.backend.opencompass.tasks.eval_api')
        self.args.config = template_config_path
        return get_config_from_arg(self.args)

    @staticmethod
    def list_datasets(return_details: bool = False):
        from opencompass.utils.run import get_config_from_arg
        from dataclasses import dataclass

        @dataclass
        class TempArgs:
            config: str
            accelerator: str = None

        template_config_path = get_module_path('evalscope.backend.opencompass.tasks.eval_api')
        template_cfg = get_config_from_arg(TempArgs(config=template_config_path))

        # e.g. ['mmlu', 'ceval', 'openai_humaneval', ...]
        dataset_show_names = list(set([_dataset['dataset_name'] for _dataset in template_cfg.datasets]))

        if return_details:
            return dataset_show_names, template_cfg.datasets
        else:
            return dataset_show_names

    def get_task_args(self):
        return self.args

    def get_cmd(self, cmd_mode: str = CmdMode.BASIC):

        if cmd_mode == CmdMode.BASIC:
            assert self.args.datasets, 'The datasets are required.'
            assert self.args.models, 'The models are required.'

            cmd_str = f'python -m run_oc ' \
                      f'--models {" ".join(self.args.models)} ' \
                      f'--datasets {" ".join(self.args.datasets)} ' \
                      f'{self.get_restore_arg("dry-run", self.args.dry_run)} ' \
                      f'{self.get_arg_with_default("work-dir", self.args.work_dir)}'

        elif cmd_mode == CmdMode.SCRIPT:
            assert self.args.config, 'The script file is required.'
            cmd_str = f'python -m run_oc {self.args.config}'
        else:
            raise ValueError(f'Unsupported command mode: {cmd_mode}')

        return cmd_str

    def run(self, run_mode: str = RunMode.FUNCTION):
        """
        The entry function to run the OpenCompass task.

        Args:
            run_mode: str, the running mode, e.g. 'function' or 'cmd'.

        Returns:
            None
        """
        if run_mode == RunMode.FUNCTION:
            from opencompass.cli.main import run_task
            from opencompass.cli.arguments import ApiModelConfig

            assert isinstance(self.args.models, list) and len(self.args.models) > 0, 'The models are required.'

            tmp_model_d: dict = self.args.models[0]
            assert 'path' in tmp_model_d and 'openai_api_base' in tmp_model_d, \
                f"Got invalid model config: {tmp_model_d}. \nTo get valid format: " \
                "{'path': 'qwen-7b-chat', 'openai_api_base': 'http://127.0.0.1:8000/v1/chat/completions'}"

            # Get valid datasets
            dataset_names = self.args.datasets       # e.g. ['mmlu', 'ceval']
            dataset_names_all, real_dataset_all = self.list_datasets(return_details=True)

            if not dataset_names:
                logger.warning(f'No datasets are specified in the config. Use all the datasets: {dataset_names_all}')
                valid_dataset_names = dataset_names_all
            else:
                valid_dataset_names, invalid_dataset_names = get_valid_list(dataset_names, dataset_names_all)
                if len(invalid_dataset_names) > 0:
                    logger.error(f'Invalid datasets: {invalid_dataset_names}, '
                                 f'refer to the following list to get proper dataset name: {dataset_names_all}')
                assert len(valid_dataset_names) > 0, f'No valid datasets. ' \
                                                     f'To get the valid datasets, please refer to {dataset_names_all}'

            valid_datasets = [_dataset for _dataset in real_dataset_all if _dataset['dataset_name'] in valid_dataset_names]
            for _dataset in valid_datasets:
                _dataset.pop('dataset_name')
                _dataset['reader_cfg']['test_range'] = self.args.limit

            # Get valid models
            models = []
            for model_d in self.args.models:
                # model_d: {'path': 'qwen-7b-chat',
                #           'meta_template': 'default-api-meta-template-oc',   # Optional
                #           'openai_api_base': 'http://127.0.0.1:8000/v1/chat/completions'}
                # Note: 'meta_template' can be a dict or a string, default is None

                if 'meta_template' in model_d and isinstance(model_d['meta_template'], str):
                    model_d['meta_template'] = get_template(model_d['meta_template'])

                # set the 'abbr' as the 'path' if 'abbr' is not specified
                model_d['abbr'] = model_d['path']

                model_config = ApiModelConfig(**model_d)
                models.append(asdict(model_config))

            # Load the initial task template and override configs
            template_cfg = self.load_task_template()
            template_cfg.datasets = valid_datasets
            template_cfg.models = models

            # Dump task config to a temporary file
            tmp_file = tempfile.NamedTemporaryFile(delete=False, suffix='.py', mode='w')
            template_cfg.dump(tmp_file.name)
            # logger.info(f'The task config is dumped to: {tmp_file.name}')
            self.args.config = tmp_file.name

            # Submit the task
            logger.info(f'*** Run task with config: {self.args.config} \n')
            run_task(self.args)

        # TODO: add more arguments for the command line
        elif run_mode == RunMode.CMD:
            subprocess.run(self.cmd, text=True, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        else:
            raise ValueError(f'Unsupported run mode: {run_mode}')


if __name__ == '__main__':

    # OpenCompassBackendManager.list_datasets()
    # ['mmlu', 'WSC', 'DRCD', 'chid', 'gsm8k', 'AX_g', 'BoolQ', 'cmnli', 'ARC_e', 'ocnli_fc', 'summedits', 'MultiRC', 'GaokaoBench', 'obqa', 'math', 'agieval', 'hellaswag', 'RTE', 'race', 'flores', 'ocnli', 'strategyqa', 'triviaqa', 'WiC', 'COPA', 'commonsenseqa', 'piqa', 'nq', 'mbpp', 'csl', 'Xsum', 'CB', 'tnews', 'ARC_c', 'afqmc', 'eprstmt', 'ReCoRD', 'bbh', 'TheoremQA', 'CMRC', 'AX_b', 'siqa', 'storycloze', 'humaneval', 'cluewsc', 'winogrande', 'lambada', 'ceval', 'bustm', 'C3', 'lcsts']

    # 'meta_template': 'default-api-meta-template-oc',
    # models: llama3-8b-instruct, qwen-7b-chat
    oc_backend_manager = OpenCompassBackendManager(
        config={'datasets': ['mmlu', 'ceval', 'ARC_c', 'gsm8k'],
                'models': [{'path': 'llama3-8b-instruct', 'openai_api_base': 'http://127.0.0.1:8000/v1/chat/completions'}],
                'limit': 5
                }
    )
    all_datasets = OpenCompassBackendManager.list_datasets()
    print(f'all_datasets: {all_datasets}')
    oc_backend_manager.run()



# Copyright (c) Alibaba, Inc. and its affiliates.

from evalscope.benchmarks.cmmlu.cmmlu_adapter import DATASET_ID, SUBJECT_MAPPING, SUBSET_LIST, CMMLUAdapter
from evalscope.benchmarks.cmmlu.cmmlu_adapter import CMMLUAdapter as DataAdapterClass
from evalscope.models.model_adapter import MultiChoiceModelAdapter as ModelAdapterClass    # noqa


from tqdm import tqdm
import pandas as pd
import os
from evalscope.backend.rag_eval.utils.tools import save_to_jsonl, save_to_tsv

from evalscope.utils.logger import get_logger

logger = get_logger()


def evaluate(model, dataloader, limit=None, output_path=""):
    """
    Evaluate the model on the dataset
    Parameters
    ----------
    model: MultiModalModel
        model to caption the image
    dataloader: torch.utils.data.Dataloader
    limit: int
        limit the number of samples to evaluate
    Returns
    -------
    dict of retrieval metrics
    """
    sample_count = 0
    dataloader = dataloader_with_indices(dataloader)
    query_caption_index = []
    total_captions = []
    total_querys = []
    for batch_images, batch_texts, inds in tqdm(dataloader):
        captions = model.encode_image(batch_images)
        querys = [text for texts in batch_texts for text in texts]

        batch_texts_image_index = [
            ind for ind, texts in zip(inds, batch_texts) for text in texts
        ]

        total_captions.extend(captions)
        total_querys.extend(querys)
        query_caption_index.extend(batch_texts_image_index)

        if limit is not None:
            # Update sample counter
            sample_count += len(batch_images)

            if sample_count >= limit:
                break

    write_file(total_querys, total_captions, query_caption_index, output_path)
    return {"convertion_successful": True, "save_path": output_path}


def write_file(query_list, corpus_list, qrels_list, output_path):
    # 处理 query_list
    query_df = pd.DataFrame(query_list, columns=["text"])
    query_df["_id"] = query_df.index
    query_df = query_df[["_id", "text"]]
    save_to_jsonl(query_df, os.path.join(output_path, "queries.jsonl"))

    # 处理 corpus_list
    corpus_df = pd.DataFrame(corpus_list, columns=["text"])
    corpus_df["_id"] = corpus_df.index
    corpus_df = corpus_df[["_id", "text"]]
    save_to_jsonl(corpus_df, os.path.join(output_path, "corpus.jsonl"))

    # 处理 qrels_list
    qrels_df = pd.DataFrame(qrels_list, columns=["corpus-id"])
    qrels_df["query-id"] = qrels_df.index
    qrels_df["score"] = 1
    qrels_df = qrels_df[["query-id", "corpus-id", "score"]]
    save_to_tsv(qrels_df, os.path.join(output_path, "qrels", "test.tsv"))

    logger.info("Write files to {}".format(output_path))
    return


def dataloader_with_indices(dataloader):
    start = 0
    for x, y in dataloader:
        end = start + len(x)
        inds = list(range(start, end))
        yield x, y, inds
        start = end



from .Classification import *
from .Clustering import *
from .PairClassification import *
from .Reranking import *
from .Retrieval import *
from .STS import *
from .CustomTask import *


CLS_CLASSIFICATION = {
    "TNews": TNews,
    "IFlyTek": IFlyTek,
    "MultilingualSentiment": MultilingualSentiment,
    "JDReview": JDReview,
    "OnlineShopping": OnlineShopping,
    "Waimai": Waimai,
}

CLS_CLUSTERING = {
    "CLSClusteringS2S": CLSClusteringFastS2S,
    "CLSClusteringP2P": CLSClusteringFastP2P,
    "ThuNewsClusteringS2S": ThuNewsClusteringFastS2S,
    "ThuNewsClusteringP2P": ThuNewsClusteringFastP2P,
}

CLS_PAIR_CLASSIFICATION = {
    "Ocnli": Ocnli,
    "Cmnli": Cmnli,
}

CLS_RERANKING = {
    "T2Reranking": T2Reranking,
    "MMarcoReranking": MMarcoReranking,
    "CMedQAv1": CMedQAv1,
    "CMedQAv2": CMedQAv2,
}

CLS_RETRIEVAL = {
    "T2Retrieval": T2Retrieval,
    "MMarcoRetrieval": MMarcoRetrieval,
    "DuRetrieval": DuRetrieval,
    "CovidRetrieval": CovidRetrieval,
    "CmedqaRetrieval": CmedqaRetrieval,
    "EcomRetrieval": EcomRetrieval,
    "MedicalRetrieval": MedicalRetrieval,
    "VideoRetrieval": VideoRetrieval,
}

CLS_STS = {
    "ATEC": ATEC,
    "BQ": BQ,
    "LCQMC": LCQMC,
    "PAWSX": PAWSX,
    "STSB": STSB,
    "AFQMC": AFQMC,
    "QBQTC": QBQTC,
}

CLS_CUSTOM = {
    "CustomRetrieval": CustomRetrieval,
}

CLS_DICT = {
    **CLS_CLASSIFICATION,
    **CLS_CLUSTERING,
    **CLS_PAIR_CLASSIFICATION,
    **CLS_RERANKING,
    **CLS_RETRIEVAL,
    **CLS_STS,
}



import os
from typing import Optional, Union
from evalscope.utils import is_module_installed, get_valid_list
from evalscope.backend.base import BackendManager
from evalscope.utils.logger import get_logger


logger = get_logger()


class RAGEvalBackendManager(BackendManager):
    def __init__(self, config: Union[str, dict], **kwargs):
        """BackendManager for VLM Evaluation Kit

        Args:
            config (Union[str, dict]): the configuration yaml-file or the configuration dictionary
        """
        super().__init__(config, **kwargs)

    @staticmethod
    def _check_env(module_name: str):
        if is_module_installed(module_name):
            logger.info(f"Check `{module_name}` Installed")
        else:
            logger.error(f"Please install `{module_name}` first")

    @staticmethod
    def run_mteb(model_args, eval_args):
        from evalscope.backend.rag_eval.cmteb import ModelArguments, EvalArguments
        from evalscope.backend.rag_eval.cmteb import one_stage_eval, two_stage_eval

        if len(model_args) > 2:
            raise ValueError("Not support multiple models yet")

        # Convert arguments to dictionary
        model_args_list = [ModelArguments(**args).to_dict() for args in model_args]
        eval_args = EvalArguments(**eval_args).to_dict()

        if len(model_args_list) == 1:
            one_stage_eval(model_args_list[0], eval_args)
        else:  # len(model_args_list) == 2
            two_stage_eval(model_args_list[0], model_args_list[1], eval_args)

    @staticmethod
    def run_ragas(testset_args, eval_args):
        from evalscope.backend.rag_eval.ragas import rag_eval
        from evalscope.backend.rag_eval.ragas.tasks import generate_testset
        from evalscope.backend.rag_eval.ragas import (
            TestsetGenerationArguments,
            EvaluationArguments,
        )

        if testset_args is not None:
            generate_testset(TestsetGenerationArguments(**testset_args))
        if eval_args is not None:
            rag_eval(EvaluationArguments(**eval_args))

    @staticmethod
    def run_clip_benchmark(args):
        from evalscope.backend.rag_eval.clip_benchmark import Arguments, evaluate

        evaluate(Arguments(**args))

    def run(self, *args, **kwargs):
        tool = self.config_d.pop("tool")
        if tool.lower() == "mteb":
            self._check_env("mteb")
            model_args = self.config_d["model"]
            eval_args = self.config_d["eval"]
            self.run_mteb(model_args, eval_args)
        elif tool.lower() == "ragas":
            self._check_env("ragas")
            testset_args = self.config_d.get("testset_generation", None)
            eval_args = self.config_d.get("eval", None)
            self.run_ragas(testset_args, eval_args)
        elif tool.lower() == "clip_benchmark":
            self._check_env("webdataset")
            self.run_clip_benchmark(self.config_d["eval"])
        else:
            raise ValueError(f"Unknown tool: {tool}")



import os
import time
from contextlib import contextmanager
from functools import partial
from threading import Thread
from typing import List, Literal, Optional, Union

import torch
from modelscope import AutoModelForCausalLM, AutoTokenizer
from pydantic import BaseModel, Field
from transformers import TextIteratorStreamer


class Usage(BaseModel):
    prompt_tokens: int = 0
    completion_tokens: int = 0
    total_tokens: int = 0


class ModelCard(BaseModel):
    id: str
    object: str = 'model'
    created: int = Field(default_factory=lambda: int(time.time()))
    owned_by: str = 'owner'
    root: Optional[str] = None
    parent: Optional[str] = None
    permission: Optional[list] = None


class ModelList(BaseModel):
    object: str = 'list'
    data: List[ModelCard] = []


class ChatMessage(BaseModel):
    role: Literal['user', 'assistant', 'system']
    content: str


class DeltaMessage(BaseModel):
    role: Optional[Literal['user', 'assistant', 'system']] = None
    content: Optional[str] = None


class ChatCompletionRequest(BaseModel):
    model: str
    messages: List[ChatMessage] | str
    temperature: Optional[float] = None
    top_p: Optional[float] = None
    max_tokens: Optional[int] = 2048
    min_tokens: Optional[int] = None
    stream: Optional[bool] = False


class ChatCompletionResponseChoice(BaseModel):
    index: int
    message: ChatMessage
    finish_reason: Literal['stop', 'length']


class ChatCompletionResponseStreamChoice(BaseModel):
    index: int
    delta: DeltaMessage
    finish_reason: Optional[Literal['stop', 'length']]


class ChatCompletionResponse(BaseModel):
    model: str
    object: Literal['chat.completion', 'chat.completion.chunk']
    choices: List[Union[ChatCompletionResponseChoice, ChatCompletionResponseStreamChoice]]
    created: Optional[int] = Field(default_factory=lambda: int(time.time()))
    usage: Optional[Usage]


class TextCompletionRequest(BaseModel):
    model: str
    prompt: str
    temperature: Optional[float] = None
    max_tokens: Optional[int] = 2048
    min_tokens: Optional[int] = None


class TextCompletionResponseChoice(BaseModel):
    index: int
    text: str
    finish_reason: Literal['stop', 'length']


class TextCompletionResponse(BaseModel):
    model: str
    object: Literal['text_completion']
    created: Optional[int] = Field(default_factory=lambda: int(time.time()))
    choices: List[TextCompletionResponseChoice]
    usage: Optional[Usage]


class ChatService:

    def __init__(self, model_path, attn_implementation):
        self.tokenizer = AutoTokenizer.from_pretrained(model_path, trust_remote_code=True)
        self.model = AutoModelForCausalLM.from_pretrained(
            model_path,
            trust_remote_code=True,
            device_map='auto',
            torch_dtype='auto',
            attn_implementation=attn_implementation,
        ).eval()
        self.streamer = TextIteratorStreamer(self.tokenizer, skip_prompt=True)
        self.model_id = os.path.basename(model_path)
        self.device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')

    def count_tokens(self, text: str) -> int:
        # Use the tokenizer to count the number of tokens
        return len(self.tokenizer.encode(text, add_special_tokens=False))

    async def list_models(self):
        model_card = ModelCard(id=self.model_id)
        return ModelList(data=[model_card])

    async def _chat(self, request: ChatCompletionRequest):
        formatted_prompt, inputs, prompt_tokens = self._prepare_chat_inputs(request)
        outputs = self.model.generate(
            **inputs,
            max_new_tokens=request.max_tokens,
            min_new_tokens=request.min_tokens,
            temperature=request.temperature,
        )
        outputs = outputs[0][prompt_tokens:]  # remove prompt
        completion_tokens = len(outputs)
        response = self.tokenizer.decode(outputs, skip_special_tokens=True)

        choice_data = ChatCompletionResponseChoice(
            index=0,
            message=ChatMessage(role='assistant', content=response),
            finish_reason='stop',
        )
        return ChatCompletionResponse(
            model=self.model_id,
            choices=[choice_data],
            object='chat.completion',
            usage=Usage(
                prompt_tokens=prompt_tokens,
                completion_tokens=completion_tokens,
                total_tokens=prompt_tokens + completion_tokens,
            ),
        )

    async def _text_completion(self, request: TextCompletionRequest):
        inputs, prompt_tokens = self._prepare_text_inputs(request)
        outputs = self.model.generate(
            **inputs,
            max_new_tokens=request.max_tokens,
            min_new_tokens=request.min_tokens,
            temperature=request.temperature,
        )
        outputs = outputs[0][prompt_tokens:]  # remove prompt
        completion_tokens = len(outputs)
        response = self.tokenizer.decode(outputs, skip_special_tokens=True)

        choice_data = TextCompletionResponseChoice(
            index=0,
            text=response,
            finish_reason='stop',
        )
        return TextCompletionResponse(
            model=self.model_id,
            choices=[choice_data],
            object='text_completion',
            usage=Usage(
                prompt_tokens=prompt_tokens,
                completion_tokens=completion_tokens,
                total_tokens=prompt_tokens + completion_tokens,
            ),
        )

    def _prepare_text_inputs(self, request: TextCompletionRequest):
        inputs = self.tokenizer(request.prompt, return_tensors='pt', padding=True).to(self.device)
        prompt_tokens = len(inputs['input_ids'][0])
        return inputs, prompt_tokens

    def _stream_chat(self, request: ChatCompletionRequest):
        formatted_prompt, inputs, prompt_tokens = self._prepare_chat_inputs(request)
        completion_tokens = 0

        yield self._create_initial_chunk()

        generation_kwargs = dict(
            **inputs,
            streamer=self.streamer,
            max_new_tokens=request.max_tokens,
            min_new_tokens=request.min_tokens,
            temperature=request.temperature,
        )
        generate_partial = partial(self.model.generate, **generation_kwargs)

        with self._start_generation_thread(generate_partial):
            for new_text in self.streamer:
                yield self._create_chunk(new_text)
                completion_tokens += self.count_tokens(new_text)

        yield self._create_final_chunk(prompt_tokens, completion_tokens)
        yield '[DONE]'

    def _prepare_chat_inputs(self, request: ChatCompletionRequest):
        formatted_prompt = self.tokenizer.apply_chat_template(
            request.messages, tokenize=False, add_generation_prompt=True)
        inputs = self.tokenizer(formatted_prompt, return_tensors='pt', padding=True).to(self.device)
        prompt_tokens = len(inputs['input_ids'][0])
        return formatted_prompt, inputs, prompt_tokens

    @contextmanager
    def _start_generation_thread(self, generate_partial):
        thread = Thread(target=generate_partial)
        thread.start()
        try:
            yield
        finally:
            thread.join()

    def _create_initial_chunk(self):
        choice_data = ChatCompletionResponseStreamChoice(index=0, delta={'role': 'assistant'}, finish_reason=None)
        chunk = ChatCompletionResponse(
            model=self.model_id,
            choices=[choice_data],
            object='chat.completion.chunk',
            usage=None,
        )
        return chunk.model_dump_json(exclude_unset=True)

    def _create_chunk(self, new_text):
        choice_data = ChatCompletionResponseStreamChoice(index=0, delta={'content': new_text}, finish_reason=None)
        chunk = ChatCompletionResponse(
            model=self.model_id,
            choices=[choice_data],
            object='chat.completion.chunk',
            usage=None,
        )
        return chunk.model_dump_json(exclude_unset=True)

    def _create_final_chunk(self, prompt_tokens, completion_tokens):
        choice_data = ChatCompletionResponseStreamChoice(index=0, delta={}, finish_reason='stop')
        chunk = ChatCompletionResponse(
            model=self.model_id,
            choices=[choice_data],
            object='chat.completion.chunk',
            usage=Usage(
                prompt_tokens=prompt_tokens,
                completion_tokens=completion_tokens,
                total_tokens=prompt_tokens + completion_tokens,
            ),
        )
        return chunk.model_dump_json(exclude_unset=True)



# Copyright (c) Alibaba, Inc. and its affiliates.
from abc import ABC, abstractmethod
from typing import Any, Union, Dict, List
import torch


class CustomModel(ABC):

    def __init__(self, config: dict, **kwargs):
        self.config = config
        self.kwargs = kwargs

        if config.get('model_id', None) is None:
            raise ValueError(f"**Error: model_id is required in config for CustomModel. Got config: {config}")

    @abstractmethod
    @torch.no_grad()
    def predict(self, prompts: List[str], **kwargs) -> List[Dict[str, Any]]:
        """
        Model prediction function for batch inputs.

        Args:
            prompts (str): The input batch of prompts to predict.

            **kwargs: kwargs

        Returns:
            res (dict): The model prediction results (batch). Format:
            [
                {
                  'choices': [
                    {
                      'index': 0,
                      'message': {
                        'content': 'xxx',
                        'role': 'assistant'
                      }
                    }
                  ],
                  'created': 1677664795,
                  'model': 'gpt-3.5-turbo-0613',   # should be model_id
                  'object': 'chat.completion',
                  'usage': {
                    'completion_tokens': 17,
                    'prompt_tokens': 57,
                    'total_tokens': 74
                  }
                }
            ,
            ...
            ]
        """
        raise NotImplementedError



import copy
import subprocess
from functools import partial
from typing import Optional, Union

from evalscope.backend.base import BackendManager
from evalscope.utils import get_valid_list, is_module_installed
from evalscope.utils.logger import get_logger

logger = get_logger()


class ExecutionMode:

    # The command mode is to run the command directly with the command line.
    CMD = 'cmd'

    # The function mode is to run the command with a function call -- run_task().
    FUNCTION = 'function'


class VLMEvalKitBackendManager(BackendManager):

    def __init__(self, config: Union[str, dict], **kwargs):
        """BackendManager for VLM Evaluation Kit

        Args:
            config (Union[str, dict]): the configuration yaml-file or the configuration dictionary
        """
        self._check_env()
        super().__init__(config, **kwargs)
        from vlmeval.utils.arguments import Arguments as VLMEvalArguments
        self.args = VLMEvalArguments(**self.config_d)

        self.valid_models = self.list_supported_models()
        self.valid_model_names = list(self.valid_models.keys())
        self.valid_datasets = self.list_supported_datasets()

        self._check_valid()

    def _check_valid(self):
        # Ensure not both model and datasets are empty
        if not self.args.data or not self.args.model:
            raise ValueError('** Args: Please provide model and datasets. **')

        # Check datasets
        valid_datasets, invalid_datasets = get_valid_list(self.args.data, self.valid_datasets)
        if len(invalid_datasets) != 0:
            logger.warning(f'Using custom dataset: {invalid_datasets}, ')

        # Check model
        if isinstance(self.args.model[0], dict):
            model_names = [model['name'] for model in self.args.model]
            valid_model_names, invalid_model_names = get_valid_list(model_names, self.valid_model_names)
            assert len(invalid_model_names) == 0, f'Invalid models: {invalid_model_names}, ' \
                f'refer to the following list to get proper model name: {self.valid_model_names}'

            # set model_cfg
            new_model_names = []
            for model_cfg in self.args.model:
                model_name = model_cfg['name']
                model_class = self.valid_models[model_name]
                if model_name == 'CustomAPIModel':
                    model_type = model_cfg['type']
                    remain_cfg = copy.deepcopy(model_cfg)
                    del remain_cfg['name']  # remove not used args
                    del remain_cfg['type']  # remove not used args

                    self.valid_models.update({model_type: partial(model_class, model=model_type, **remain_cfg)})
                    new_model_names.append(model_type)
                else:
                    remain_cfg = copy.deepcopy(model_cfg)
                    del remain_cfg['name']  # remove not used args

                    self.valid_models[model_name] = partial(model_class, **remain_cfg)
                    new_model_names.append(model_name)

            self.args.model = new_model_names

        elif isinstance(self.args.model[0], str):
            valid_model_names, invalid_model_names = get_valid_list(self.args.model, self.valid_model_names)
            if len(invalid_datasets) != 0:
                logger.warning(f'Using custom dataset: {invalid_datasets}, ')

    @property
    def cmd(self):
        return self.get_cmd()

    @staticmethod
    def list_supported_models():
        from vlmeval.config import supported_VLM
        return supported_VLM

    @staticmethod
    def list_supported_datasets():
        from vlmeval.dataset import SUPPORTED_DATASETS
        return SUPPORTED_DATASETS

    @staticmethod
    def _check_env():
        if is_module_installed('vlmeval'):
            logger.info('Check VLM Evaluation Kit: Installed')

    @staticmethod
    def get_restore_arg(arg_name: str, arg_val: bool):
        if arg_val:
            return f'--{arg_name}'
        else:
            return ''

    @staticmethod
    def get_arg_with_default(arg_name: str, arg_val: Optional[str] = None):
        if arg_val:
            return f'--{arg_name} {arg_val}'
        else:
            return ''

    def get_cmd(self):
        assert self.args.data, 'The datasets are required.'
        assert self.args.model, 'The models are required.'

        cmd_str = f'python -m vlmeval ' \
            f'--model {" ".join(self.args.model)} ' \
            f'--data {" ".join(self.args.data)} ' \
            f'{self.get_restore_arg("verbose", self.args.verbose)} ' \
            f'{self.get_restore_arg("ignore", self.args.ignore)} ' \
            f'{self.get_restore_arg("reuse", self.args.reuse)} ' \
            f'{self.get_arg_with_default("work-dir", self.args.work_dir)} ' \
            f'{self.get_arg_with_default("limit", self.args.limit)} ' \
            f'{self.get_arg_with_default("mode", self.args.mode)} ' \
            f'{self.get_arg_with_default("nproc", self.args.nproc)} ' \
            f'{self.get_arg_with_default("judge", self.args.judge)} ' \
            f'{self.get_arg_with_default("retry", self.args.retry)} '

        return cmd_str

    def run(self, run_mode: str = ExecutionMode.FUNCTION):
        if run_mode == ExecutionMode.CMD:
            logger.info(f'** Run command: {self.cmd}')
            try:
                subprocess.run(
                    self.cmd,
                    check=True,
                    ext=True,
                    shell=True,
                )
            except subprocess.CalledProcessError as e:
                logger.error(f'** Run command failed: {e.stderr}')
                raise

        elif run_mode == ExecutionMode.FUNCTION:
            from vlmeval.run import run_task
            logger.info(f'*** Run task with config: {self.args} \n')
            run_task(self.args)

        else:
            raise NotImplementedError



from evalscope.backend.rag_eval.utils.embedding import EmbeddingModel
from evalscope.backend.rag_eval.utils.llm import LLM, LocalLLM, ChatOpenAI
from evalscope.backend.rag_eval.utils.clip import VisionModel
from evalscope.backend.rag_eval.backend_manager import RAGEvalBackendManager



# Copyright (c) Alibaba, Inc. and its affiliates.

import os
import subprocess
import time
import unittest

import json
import requests

from evalscope.backend.opencompass import OpenCompassBackendManager
from evalscope.run import run_task
from evalscope.summarizer import Summarizer
from evalscope.utils import is_module_installed, test_level_list
from evalscope.utils.logger import get_logger

logger = get_logger(__name__)

DEFAULT_CHAT_MODEL_URL = 'http://127.0.0.1:8000/v1/chat/completions'
DEFAULT_BASE_MODEL_URL = 'http://127.0.0.1:8001/v1/completions'


class TestRunSwiftEval(unittest.TestCase):

    def setUp(self) -> None:
        logger.info('Init env for swift-eval UTs ...\n')

        self.model_name = 'qwen2_5-0_5b-instruct'
        assert is_module_installed('evalscope'), 'Please install `evalscope` from pypi or source code.'

        if not is_module_installed('opencompass'):
            logger.warning('Note: installing ms-opencompass ...')
            subprocess.run('pip3 install ms-opencompass -U', shell=True, check=True)

        if not is_module_installed('swift'):
            logger.warning('Note: installing ms-swift ...')
            subprocess.run('pip3 install ms-swift[llm]', shell=True, check=True)

        logger.warning('vllm not installed, use native swift deploy service instead.')

        logger.info('\nStaring run swift deploy ...')
        self.process_swift_deploy = subprocess.Popen(
            f'swift deploy --model_type {self.model_name} --infer_backend pt', text=True, shell=True)
        if self.process_swift_deploy.stderr:
            logger.info(f'swift deploy log info: {self.process_swift_deploy.stderr}')

        self.all_datasets = OpenCompassBackendManager.list_datasets()
        assert len(self.all_datasets) > 0, f'Failed to list datasets from OpenCompass backend: {self.all_datasets}'

    def tearDown(self) -> None:
        # Stop the swift deploy model service
        logger.warning('\nStopping swift deploy ...')
        self.process_swift_deploy.terminate()
        self.process_swift_deploy.wait()
        logger.info('Process swift-deploy terminated successfully.')

    @staticmethod
    def find_and_kill_pid(pids: list):
        if len(pids) > 0:
            for pid in pids:
                subprocess.run(['kill', str(pid)])
                logger.warning(f'Killed process {pid}.')
        else:
            logger.info('No pids found.')

    @staticmethod
    def find_and_kill_service(service_name):
        try:
            # find pid
            result = subprocess.run(['ps', '-ef'], stdout=subprocess.PIPE, text=True)

            lines = result.stdout.splitlines()
            pids = []
            for line in lines:
                if service_name in line and 'grep' not in line:
                    parts = line.split()
                    pid = parts[1]
                    pids.append(pid)

            if not pids:
                logger.info(f'No process found for {service_name}.')
            else:
                for pid in pids:
                    subprocess.run(['kill', pid])
                    logger.warning(f'Killed process {pid} for service {service_name}.')
        except Exception as e:
            logger.error(f'An error occurred: {e}')

    @staticmethod
    def check_service_status(url: str, data: dict, retries: int = 30, delay: int = 10):
        for i in range(retries):
            try:
                logger.info(f'Attempt {i + 1}: Checking service at {url} ...')
                response = requests.post(
                    url, data=json.dumps(data), headers={'Content-Type': 'application/json'}, timeout=30)
                if response.status_code == 200:
                    logger.info(f'Service at {url} is available !\n\n')
                    return True
                else:
                    logger.info(f'Service at {url} returned status code {response.status_code}.')
            except requests.exceptions.RequestException as e:
                logger.info(f'Attempt {i + 1}: An error occurred: {e}')

            time.sleep(delay)

        logger.info(f'Service at {url} is not available after {retries} retries.')
        return False

    @unittest.skipUnless(1 in test_level_list(), 'skip test in current test level')
    def test_run_task(self):
        # Prepare the config
        task_cfg = dict(
            eval_backend='OpenCompass',
            eval_config={
                'datasets': ['cmb', 'bbh', 'ceval', 'ARC_e', 'gsm8k'],
                'models': [
                    {
                        'path': self.model_name,
                        'openai_api_base': DEFAULT_CHAT_MODEL_URL,
                        'batch_size': 8
                    },
                ],
                'work_dir': 'outputs/llama3_eval_result',
                'reuse': None,  # string, `latest` or timestamp, e.g. `20230516_144254`, default to None
                'limit': 5,  # string or int or float, e.g. `[2:5]`, 5, 5.0, default to None, it means run all examples
            },
        )

        # Check the service status
        data = {'model': self.model_name, 'messages': [{'role': 'user', 'content': 'who are you?'}]}
        assert self.check_service_status(
            DEFAULT_CHAT_MODEL_URL, data=data), f'Failed to check service status: {DEFAULT_CHAT_MODEL_URL}'

        # Submit the task
        logger.info(f'Start to run UT with cfg: {task_cfg}')
        run_task(task_cfg=task_cfg)

        # Get the final report with summarizer
        report_list = Summarizer.get_report_from_cfg(task_cfg)
        logger.info(f'>>The report list:\n{report_list}')

        assert len(report_list) > 0, f'Failed to get report list: {report_list}'


if __name__ == '__main__':
    unittest.main()



# Copyright (c) Alibaba, Inc. and its affiliates.
import os
import unittest

from evalscope.perf.main import run_perf_benchmark
from evalscope.utils import test_level_list


class TestPerf(unittest.TestCase):

    def setUp(self) -> None:
        pass

    def tearDown(self) -> None:
        pass

    @unittest.skipUnless(0 in test_level_list(), 'skip test in current test level')
    def test_run_perf(self):
        task_cfg = {
            'url': 'http://127.0.0.1:8000/v1/chat/completions',
            'parallel': 1,
            'model': 'qwen2.5',
            'number': 15,
            'api': 'openai',
            'dataset': 'openqa',
            'debug': True,
        }
        run_perf_benchmark(task_cfg)

    @unittest.skipUnless(0 in test_level_list(), 'skip test in current test level')
    def test_run_perf_stream(self):
        task_cfg = {
            'url': 'http://127.0.0.1:8000/v1/chat/completions',
            'parallel': 1,
            'model': 'qwen2.5',
            'number': 15,
            'api': 'openai',
            'dataset': 'openqa',
            'stream': True,
            'debug': True,
        }
        run_perf_benchmark(task_cfg)

    @unittest.skipUnless(0 in test_level_list(), 'skip test in current test level')
    def test_run_perf_speed_benchmark(self):
        task_cfg = {
            'url': 'http://127.0.0.1:8000/v1/completions',
            'parallel': 1,
            'model': 'qwen2.5',
            'api': 'openai',
            'dataset': 'speed_benchmark',
            'debug': True,
        }
        run_perf_benchmark(task_cfg)

    @unittest.skipUnless(0 in test_level_list(), 'skip test in current test level')
    def test_run_perf_local(self):
        task_cfg = {
            'parallel': 1,
            'model': 'Qwen/Qwen2.5-0.5B-Instruct',
            'number': 5,
            'api': 'local',
            'dataset': 'openqa',
            'debug': True,
        }
        run_perf_benchmark(task_cfg)

    @unittest.skipUnless(0 in test_level_list(), 'skip test in current test level')
    def test_run_perf_local_stream(self):
        task_cfg = {
            'parallel': 1,
            'model': 'Qwen/Qwen2.5-0.5B-Instruct',
            'number': 5,
            'api': 'local',
            'dataset': 'openqa',
            'stream': True,
            'debug': True,
        }
        run_perf_benchmark(task_cfg)

    @unittest.skipUnless(0 in test_level_list(), 'skip test in current test level')
    def test_run_perf_local_speed_benchmark(self):
        task_cfg = {
            'parallel': 1,
            'model': 'Qwen/Qwen2.5-0.5B-Instruct',
            'api': 'local_vllm',
            'dataset': 'speed_benchmark',
            'min_tokens': 2048,
            'max_tokens': 2048,
            'debug': True,
        }
        run_perf_benchmark(task_cfg)


if __name__ == '__main__':
    unittest.main(buffer=False)



import base64
import pickle
import sqlite3

import json

result_db_path = '/mnt/data/data/user/maoyunlin.myl/eval-scope/outputs/qwen2.5_benchmark_20241111_160543.db'
con = sqlite3.connect(result_db_path)
query_sql = "SELECT request, response_messages, prompt_tokens, completion_tokens \
                FROM result WHERE success='1'"

# how to save base64.b64encode(pickle.dumps(benchmark_data["request"])).decode("ascii"),
with con:
    rows = con.execute(query_sql).fetchall()
    if len(rows) > 0:
        for row in rows:
            request = row[0]
            responses = row[1]
            request = base64.b64decode(request)
            request = pickle.loads(request)
            responses = base64.b64decode(responses)
            responses = pickle.loads(responses)
            response_content = ''
            for response in responses:
                response = json.loads(response)
                if not response['choices']:
                    continue
                response_content += response['choices'][0]['delta']['content']
            print('prompt: %s, tokens: %s, completion: %s, tokens: %s' %
                  (request['messages'][0]['content'], row[2], response_content, row[3]))



# Copyright (c) Alibaba, Inc. and its affiliates.
# Copyright (c) ZhipuAI, Inc. and its affiliates.

import os
import json
from typing import List

import torch
import numpy as np
import random
from modelscope import AutoTokenizer, AutoModelForCausalLM
from tqdm import tqdm

from evalscope.third_party.longbench_write.utils import count_words
from evalscope.models.api import OpenaiApi
from evalscope.utils import get_logger

logger = get_logger()

DEFAULT_PROC_NUM = 8

"""
This script is used to generate predictions for the LongWriter model.
Refer to https://github.com/THUDM/LongWriter for more details.
"""


def get_pred(rank, world_size, data, path, max_new_tokens, temperature, tokenizer, fout):
    device = torch.device(f'cuda:{rank}')
    model = AutoModelForCausalLM.from_pretrained(path, trust_remote_code=True, torch_dtype=torch.bfloat16).to(device)
    model = model.eval()

    for dt in tqdm(data, total=len(data), desc=f'Infer on rank-{rank}: '):
        prompt = dt['prompt']
        if "llama" in path.lower():
            prompt = f"[INST]{prompt}[/INST]"
            input = tokenizer(prompt, truncation=False, return_tensors="pt").to(device)
            context_length = input.input_ids.shape[-1]
            output = model.generate(
                **input,
                max_new_tokens=max_new_tokens,
                num_beams=1,
                do_sample=True,
                temperature=temperature,
            )[0]
            response = tokenizer.decode(output[context_length:], skip_special_tokens=True)
        else:
            response, history = model.chat(tokenizer, prompt, history=[], max_new_tokens=max_new_tokens,
                                           temperature=temperature)
        dt["response_length"], _ = count_words(response)
        dt["response"] = response

        logger.info(dt)

        fout.write(json.dumps(dt, ensure_ascii=False) + '\n')
        fout.flush()

    logger.info(f'Successfully generated predictions for {len(data)} samples.')


def seed_everything(seed):
    torch.manual_seed(seed)
    torch.cuda.manual_seed(seed)
    np.random.seed(seed)
    random.seed(seed)
    torch.backends.cudnn.benchmark = False
    torch.backends.cudnn.deterministic = True
    torch.cuda.manual_seed_all(seed)


# def run_infer(model: str,
#               data_path: str,
#               output_dir: str,
#               generation_kwargs: dict = None,
#               enable: bool = True, ):
#     """
#     Process inference for LongWriter model.
#
#     Args:
#         model: The model id of the LongWriter model on ModelScope, or local model path.
#         data_path: The path to the data file.
#         output_dir: The output directory for the predictions.
#         generation_kwargs: The generation arguments for the model.
#             Attributes: `max_new_tokens`: The maximum number of tokens to generate. `temperature`: The temperature
#         enable: Whether to run infer process.
#     """
#     model_id_path: str = os.path.join(output_dir, model.strip(os.sep).replace(os.sep, '__'))
#
#     if not enable:
#         logger.warning('*** Skip `infer` stage ***')
#         return f'{model_id_path}/pred.jsonl'
#
#     seed_everything(42)
#
#     os.makedirs(model_id_path, exist_ok=True)
#     fout = open(f'{model_id_path}/pred.jsonl', 'w', encoding='utf-8')
#
#     if generation_kwargs is None:
#         generation_kwargs = dict({
#             'max_new_tokens': 32768,
#             'temperature': 0.5
#         })
#
#     tokenizer = AutoTokenizer.from_pretrained(model, trust_remote_code=True)
#     world_size = torch.cuda.device_count()
#
#     logger.info(f'>>Input data path: {data_path}')
#     with open(data_path, encoding='utf-8') as f:
#         data = [json.loads(line) for line in f]
#
#     data_subsets = [data[i::world_size] for i in range(world_size)]
#     processes = []
#     for rank in range(world_size):
#         p = mp.Process(target=get_pred,
#                        args=(rank, world_size, data_subsets[rank], model, generation_kwargs.get('max_new_tokens'), generation_kwargs.get('temperature'), tokenizer, fout))
#         p.start()
#         processes.append(p)
#
#     for p in processes:
#         p.join()
#
#     logger.info(f'Finish generating predictions for {model}.')
#     logger.info(f'Predictions are saved in {model_id_path}/pred.jsonl.')
#
#     return f'{model_id_path}/pred.jsonl'


def run_infer(model: str,
              data_path: str,
              output_dir: str,
              api_config: dict,
              generation_kwargs: dict = None,
              enable: bool = True, ):
    """
    Process inference for LongWriter model.

    Args:
        model: The model id of the LongWriter model on ModelScope, or local model path.
        data_path: The path to the data file.
        output_dir: The output directory for the predictions.
        api_config: The configuration for the OpenAI API inference.
            Attributes:
                `openai_api_key`: The OpenAI API key. Default is None for custom model serving.
                `openai_api_base`: The OpenAI API base URL.
                `is_chat`: Whether to chat. Default is True.
                `verbose`: Whether to print verbose information. Default is False.
        generation_kwargs: The generation arguments for the model.
            Attributes: `max_new_tokens`: The maximum number of tokens to generate. `temperature`: The temperature
        enable: Whether to run infer process.
    """
    model_id_path: str = os.path.join(output_dir, model.strip(os.sep).replace(os.sep, '__'))

    if not enable:
        logger.warning('*** Skip `infer` stage ***')
        return f'{model_id_path}/pred.jsonl'

    seed_everything(42)

    if generation_kwargs is None:
        generation_kwargs = dict({
            'max_new_tokens': 32768,
            'temperature': 0.5,
            'repetition_penalty': 1.0,
        })

    # Prepare inputs
    logger.info(f'>>Input data path: {data_path}')
    # TODO: add load data from ms
    with open(data_path, encoding='utf-8') as f:
        data_list = [json.loads(line) for line in f]

    logger.info(f'Input example: {data_list[0]}')

    api_client = OpenaiApi(model=model,
                           openai_api_key=None,
                           openai_api_base=api_config.get('openai_api_base', 'http://127.0.0.1:8000/v1/chat/completions'),
                           max_new_tokens=generation_kwargs.get('max_new_tokens', 4096),
                           temperature=generation_kwargs.get('temperature', 0.0),
                           repetition_penalty=generation_kwargs.get('repetition_penalty', 1.0),
                           is_chat=api_config.get('is_chat', True),
                           verbose=api_config.get('verbose', False),
                           )

    # TODO: ONLY FOR TEST  generate_simple
    results: List[str] = api_client.generate_simple(inputs=[example['prompt'] for example in data_list])
    assert len(results) == len(data_list), f'Error: The number of predictions {len(results)} is not equal to the number of inputs {len(data_list)}.'
    logger.info(f'Finish generating predictions with {len(data_list)} samples for {model}')

    # Outputs
    os.makedirs(model_id_path, exist_ok=True)
    output_pred_file: str = f'{model_id_path}/pred.jsonl'
    with open(output_pred_file, 'w', encoding='utf-8') as f:
        for dt, res in zip(data_list, results):
            dt["response_length"], _ = count_words(res)
            dt["response"] = res
            f.write(json.dumps(dt, ensure_ascii=False) + '\n')

    logger.info(f'Predictions are saved in {output_pred_file}')

    return output_pred_file


if __name__ == '__main__':
    # ZhipuAI/LongWriter-glm4-9b, ZhipuAI/LongWriter-llama3.1-8b
    api_config = dict(openai_api_key=None,
                      openai_api_base='http://127.0.0.1:8000/v1/chat/completions',
                      is_chat=True,
                      verbose=True,)

    run_infer(model='ZhipuAI/LongWriter-glm4-9b',
              data_path='resources/longbench_write.jsonl',
              output_dir='outputs',
              api_config=api_config,
              generation_kwargs=dict({
                      'max_new_tokens': 32768,
                      'temperature': 0.5})
              )



# Copyright (c) Alibaba, Inc. and its affiliates.

from evalscope.benchmarks.race.race_adapter import DATASET_ID, SUBJECT_MAPPING, SUBSET_LIST, RACEAdapter
from evalscope.benchmarks.race.race_adapter import RACEAdapter as DataAdapterClass
from evalscope.models.model_adapter import MultiChoiceModelAdapter as ModelAdapterClass    # noqa



# Copyright (c) Alibaba, Inc. and its affiliates.
import os
from typing import Union
from copy import deepcopy

from evalscope.third_party.toolbench_static.infer import InferArgs, run_infer
from evalscope.third_party.toolbench_static.eval import EvalArgs, run_eval
from evalscope.utils import yaml_to_dict, get_logger, json_to_dict

logger = get_logger()


def run_task(task_cfg: Union[str, dict]):

    if isinstance(task_cfg, str):
        if task_cfg.endswith('.yaml'):
            task_cfg: dict = yaml_to_dict(task_cfg)
        elif task_cfg.endswith('.json'):
            task_cfg: dict = json_to_dict(task_cfg)
        else:
            raise ValueError(f'Unsupported file format: {task_cfg}, should be yaml or json file.')

    # Run inference for each domain
    infer_args: dict = task_cfg['infer_args']
    for domain in ['in_domain', 'out_of_domain']:
        domain_infer_args = deepcopy(infer_args)
        domain_infer_args.update({'data_path': os.path.join(infer_args['data_path'], f'{domain}.json')})
        domain_infer_args.update({'output_dir': os.path.join(infer_args['output_dir'], domain)})

        task_infer_args = InferArgs(**domain_infer_args)
        print(f'**Run infer config: {task_infer_args}')
        run_infer(task_infer_args)

    # Run evaluation for each domain
    eval_args: dict = task_cfg['eval_args']
    for domain in ['in_domain', 'out_of_domain']:
        domain_eval_args = deepcopy(eval_args)
        domain_eval_args.update({'input_path': os.path.join(eval_args['input_path'], domain)})
        domain_eval_args.update({'output_path': os.path.join(eval_args['output_path'], domain)})

        task_eval_args = EvalArgs(**domain_eval_args)
        print(f'**Run eval config: {task_eval_args}')
        run_eval(task_eval_args)


if __name__ == '__main__':
    # task_cfg_file = 'config_default.yaml'
    task_cfg_file = 'config_default.json'

    run_task(task_cfg=task_cfg_file)



from typing import Any, Dict, Iterator, List

from modelscope import MsDataset

from evalscope.perf.arguments import Arguments
from evalscope.perf.plugin.datasets.base import DatasetPluginBase
from evalscope.perf.plugin.registry import register_dataset


@register_dataset('longalpaca')
class LongAlpacaDatasetPlugin(DatasetPluginBase):
    """Read data from file which is list of requests.
           Sample: https://www.modelscope.cn/datasets/AI-ModelScope/LongAlpaca-12k/files
    """

    def __init__(self, query_parameters: Arguments):
        super().__init__(query_parameters)

    def build_messages(self) -> Iterator[List[Dict]]:
        if not self.query_parameters.dataset_path:
            ds = MsDataset.load('AI-ModelScope/LongAlpaca-12k', subset_name='default', split='train')
        else:
            ds = self.dataset_json_list(self.query_parameters.dataset_path)
        for item in ds:
            prompt = item['instruction'].strip()
            if len(prompt) > self.query_parameters.min_prompt_length and len(
                    prompt) < self.query_parameters.max_prompt_length:
                yield [{'role': 'user', 'content': prompt}]



import os
import torch
import json
from itertools import product

from evalscope.backend.rag_eval.clip_benchmark.dataset_builder import (
    build_dataset,
    get_dataset_default_task,
    get_dataloader,
)
from evalscope.backend.rag_eval.clip_benchmark.tasks import (
    zeroshot_classification,
    zeroshot_retrieval,
    image_caption,
)
from evalscope.backend.rag_eval.clip_benchmark.arguments import Arguments
from evalscope.backend.rag_eval.utils.clip import VisionModel
from evalscope.utils.logger import get_logger

logger = get_logger()


def evaluate(args: Arguments):
    models = args.models
    dataset_names = args.dataset_name
    data_dir = args.data_dir
    split = args.split
    batch_size = args.batch_size
    num_workers = args.num_workers
    verbose = args.verbose
    input_task = args.task
    output_dir = args.output_dir
    cache_dir = args.cache_dir
    skip_existing = args.skip_existing
    limit = args.limit

    # Iterate over model and dataset combinations
    for model_cfg, dataset_name in product(models, dataset_names):
        task = input_task or get_dataset_default_task(dataset_name)
        model_name = os.path.basename(model_cfg["model_name"])

        output_path = os.path.join(output_dir, model_name)
        os.makedirs(output_path, exist_ok=True)
        output_file = os.path.join(output_path, f"{dataset_name}_{task}.json")

        # Skip evaluation if the result already exists and skip_existing is True
        if os.path.exists(output_file) and skip_existing:
            if verbose:
                logger.info(f"Skip {output_dir}, exists already.")
            return

        # Determine device (CPU or GPU)
        device = "cuda" if torch.cuda.is_available() else "cpu"
        model_cfg["device"] = device
        # Initialize the model
        model = VisionModel.load(**model_cfg)

        # Build the dataset
        dataset = build_dataset(
            dataset_name=dataset_name,
            root=data_dir,
            transform=model.transform,
            split=split,
            wds_cache_dir=f"{cache_dir}/{dataset_name}",
        )

        # Create the dataloader
        dataloader = get_dataloader(dataset_name, dataset, batch_size, num_workers)

        # Evaluate based on the task
        if task == "zeroshot_classification":
            zeroshot_templates = (
                dataset.templates if hasattr(dataset, "templates") else None
            )
            if verbose:
                logger.info(f"Zero-shot templates: {zeroshot_templates}")
            classnames = dataset.classes if hasattr(dataset, "classes") else None
            assert (
                zeroshot_templates is not None and classnames is not None
            ), "Dataset does not support classification"
            metrics = zeroshot_classification.evaluate(
                model,
                dataloader,
                classnames,
                zeroshot_templates,
                device=device,
                verbose=verbose,
                limit=limit,
            )
        elif task == "zeroshot_retrieval":
            metrics = zeroshot_retrieval.evaluate(
                model, dataloader, recall_k_list=[5], device=device, limit=limit
            )
        elif task == "image_caption":
            output_path = os.path.join(output_path, dataset_name, "retrieval_data")
            metrics = image_caption.evaluate(
                model, dataloader, limit=limit, output_path=output_path
            )

        # Prepare dump data
        dump = {
            "dataset": dataset_name,
            "model": model_name,
            "task": task,
            "metrics": metrics,
        }

        if verbose:
            logger.info(f"Evaluation results: {dump}")

        # Write the results to output file
        if verbose:
            logger.info(f"Dump results to: {output_file}")
        with open(output_file, "w") as f:
            json.dump(dump, f)


if __name__ == "__main__":
    evaluate()



# Copyright (c) Alibaba, Inc. and its affiliates.

import os
import json
import glob
from tabulate import tabulate
from evalscope.utils.logger import get_logger

logger = get_logger()

"""
Combine and generate table for reports of LLMs.
"""


def get_report(report_file: str):
    data_d: dict = json.load(open(report_file, 'r'))
    dataset_name = data_d['name']
    score = data_d['score']     # float or dict
    score_d = {}
    if isinstance(score, dict):
        # score_d = dict([(k, round(v, 4) * 100) for k, v in score.items()])
        score_d = score
    elif isinstance(score, float):
        # score_d['acc'] = round(score, 4) * 100
        score_d['acc'] = score
    else:
        raise ValueError(f'Unknown score type: {type(score)}')
    # score_str = '\n'.join([str(v) + ' (' + k + ')' for k, v in score_d.items()])
    score_str = '\n'.join(['(' + dataset_name + '/' + k + ') ' + str(v) for k, v in score_d.items()])

    return {'dataset_name': dataset_name, 'score': score_str}


def get_model_reports(model_report_dir: str):
    model_report_dir = os.path.normpath(model_report_dir)
    model_report_dir = model_report_dir.rstrip('reports')
    model_info = os.path.basename(os.path.normpath(model_report_dir))
    model_name = '_'.join(model_info.split('_')[:-1][3:])
    report_files = glob.glob(os.path.join(model_report_dir, 'reports', '*.json'))

    model_reports_d = {model_name: []}
    for file_path in report_files:
        report_d = get_report(file_path)
        model_reports_d[model_name].append(report_d)

    return model_reports_d


def gen_table(reports_path_list: list):
    table_values = []
    headers = ['Model']
    is_headers_set = False

    for report_path in reports_path_list:
        model_reports_d = get_model_reports(report_path)
        for model_name, report_list in model_reports_d.items():
            # report_list: [{'dataset_name': 'CompetitionMath', 'score': '4.42 (acc)'},
            #               {'dataset_name': 'GSM8K', 'score': '28.51 (acc)'}]
            report_list = sorted(report_list, key=lambda x: x['dataset_name'])
            if not is_headers_set:
                headers.extend([x['dataset_name'] for x in report_list])
                is_headers_set = True
            single_row = []
            single_row.append(model_name)
            for single_report in report_list:
                # e.g. '28.51 (acc)'
                single_row.append(single_report['score'])
            table_values.append(single_row)

    report_table = tabulate(table_values, headers=headers, tablefmt='grid')
    return report_table

class ReportsRecorder:
    COMMON_DATASET_PATH = []
    CUSTOM_DATASET_PATH = []

    def __init__(self, oss_url: str = "", endpoint: str = ""):
        if oss_url and endpoint:
            import oss2
            from oss2.credentials import EnvironmentVariableCredentialsProvider

            auth = oss2.ProviderAuth(EnvironmentVariableCredentialsProvider())
            oss_url = oss_url.replace("oss://", "").split('/')
            bucket_name = oss_url[0]

            self.object_path = "/".join(oss_url[1:])
            self.bucket = oss2.Bucket(auth, endpoint, bucket_name)
        else:
            self.object_path = ""
            self.bucket = None


    def append_path(self, report_path: str, dataset_name: str):
        if dataset_name == "general_qa":
            self.CUSTOM_DATASET_PATH.append(report_path)
        else:
            self.COMMON_DATASET_PATH.append(report_path)
    
    def dump_reports(self, output_dir: str):
        result = {
            "CommonDataset": [],
            "CustomDataset": []
        }
        for line in self.COMMON_DATASET_PATH:
            with open(line, 'r') as f:
                report = json.load(f)
                result['CommonDataset'].append(report)
        for line in self.CUSTOM_DATASET_PATH:
            with open(line, 'r') as f:
                report = json.load(f)
                report.update({"name": os.path.basename(line)})
                result['CustomDataset'].append(report)
        
        os.makedirs(output_dir, exist_ok=True)
        output_file_name = "metric.json"
        output_path = os.path.join(output_dir, output_file_name)
        with open(output_path, 'w+') as f:
            f.write(json.dumps(result, ensure_ascii=False, indent=4))
        
        if self.bucket:
            remote_path = os.path.join(self.object_path, output_file_name)
            logger.info(f"** Upload report to oss: {remote_path}")
            self.bucket.put_object_from_file(remote_path, output_path)

if __name__ == '__main__':
    report_dir_1 = '/to/path/20231129_020533_default_ZhipuAI_chatglm2-6b-base_none/reports'
    report_dir_2 = '/to/path/20231129_020533_default_ZhipuAI_chatglm2-6b_none/reports'

    report_table = gen_table([report_dir_1, report_dir_2])
    print(report_table)

    # ALL VALUES ONLY FOR EXAMPLE
    # +--------------------------+-------------------+-------------+
    # | Model                    | CompetitionMath   | GSM8K       |
    # +==========================+===================+=============+
    # | ZhipuAI_chatglm2-6b-base | 25.0 (acc)        | 30.50 (acc) |
    # +--------------------------+-------------------+-------------+
    # | ZhipuAI_chatglm2-6b      | 30.5 (acc)        | 40.50 (acc) |
    # +--------------------------+-------------------+-------------+



import io
import os
import base64
from modelscope import snapshot_download
from evalscope.utils.logger import get_logger

logger = get_logger()


def PIL_to_bytes(image_format, **kwargs):
    OPTIONS = {
        "webp": dict(format="webp", lossless=True),
        "png": dict(format="png"),
        "jpg": dict(format="jpeg"),
    }

    def transform(image):
        bytestream = io.BytesIO()
        image.save(bytestream, **OPTIONS[image_format])
        return bytestream.getvalue()

    return transform


def PIL_to_base64(image, **kwargs):
    bytestream = io.BytesIO()
    image.save(bytestream, format="jpeg")
    return base64.b64encode(bytestream.getvalue()).decode("utf-8")


def path_to_bytes(filepath):
    with open(filepath, "rb") as fp:
        return fp.read()


def path_to_base64(filepath):
    file_content = path_to_bytes(filepath)
    return base64.b64encode(file_content).decode("utf-8")


def ensure_dir(file_path):
    os.makedirs(os.path.dirname(file_path), exist_ok=True)


def save_to_jsonl(df, file_path):
    ensure_dir(file_path)
    df.to_json(file_path, orient="records", lines=True, force_ascii=False)


def save_to_tsv(df, file_path):
    ensure_dir(file_path)
    df.to_csv(file_path, sep="\t", index=False)


def download_model(model_id: str, revision: str):
    """
    default base dir: '~/.cache/modelscope/hub/model_id'
    """
    logger.info(f"Loading model {model_id} from modelscope")

    model_path = snapshot_download(model_id=model_id, revision=revision)

    return model_path



# Copyright (c) Alibaba, Inc. and its affiliates.
import json
import os
import glob
from typing import List, Union

from evalscope.config import TaskConfig
from evalscope.constants import OutputsStructure
from evalscope.tools.combine_reports import gen_table
from evalscope.utils import process_outputs_structure, yaml_to_dict, EvalBackend, json_to_dict, get_latest_folder_path, \
    csv_to_list
from evalscope.utils.logger import get_logger

logger = get_logger()


class Summarizer:

    @staticmethod
    def get_report(outputs_dir: str) -> List[dict]:
        res_list: list = []

        outputs_structure: dict = process_outputs_structure(outputs_dir, is_make=False)
        reports_dir: str = outputs_structure.get(OutputsStructure.REPORTS_DIR)
        if reports_dir is None:
            raise ValueError(f'No reports directory in {outputs_dir}')

        report_files: list = glob.glob(os.path.join(reports_dir, '*.json'))
        for report_file in report_files:
            with open(report_file, 'r') as f:
                res_list.append(json.load(f))

        report_table: str = gen_table([reports_dir])
        logger.info(f'*** Report table ***\n{report_table}')

        return res_list

    @staticmethod
    def get_report_from_cfg(task_cfg: Union[str, List[str], TaskConfig, List[TaskConfig], dict]) -> List[dict]:
        """
        Get report from cfg file.

        Args:
            task_cfg: task cfg file path. refer to evalscope/tasks/eval_qwen-7b-chat_v100.yaml

        Returns:
            list: list of report dict.
            A report dict is overall report on a benchmark for specific model.
        """
        final_res_list: List[dict] = []
        candidate_task_cfgs: List[dict] = []

        if isinstance(task_cfg, dict):
            candidate_task_cfgs = [task_cfg]
        elif isinstance(task_cfg, str):
            task_cfg: dict = yaml_to_dict(task_cfg)
            candidate_task_cfgs = [task_cfg]
        elif isinstance(task_cfg, TaskConfig):
            task_cfg: dict = task_cfg.to_dict()
            candidate_task_cfgs = [task_cfg]
        elif isinstance(task_cfg, list):
            for task_cfg_item in task_cfg:
                if isinstance(task_cfg_item, str):
                    task_cfg_item: dict = yaml_to_dict(task_cfg_item)
                elif isinstance(task_cfg_item, TaskConfig):
                    task_cfg_item: dict = task_cfg_item.to_dict()
                candidate_task_cfgs.append(task_cfg_item)
        else:
            raise ValueError(f'Invalid task_cfg: {task_cfg}')

        for candidate_task in candidate_task_cfgs:
            logger.info(f'**Loading task cfg for summarizer: {candidate_task}')
            eval_backend = candidate_task.get('eval_backend') or EvalBackend.NATIVE.value

            if eval_backend == EvalBackend.NATIVE.value:
                outputs_dir: str = candidate_task.get('outputs')
                outputs_dir: str = os.path.expanduser(outputs_dir)
                if outputs_dir is None:
                    raise ValueError(f'No outputs_dir in {task_cfg}')
                res_list: list = Summarizer.get_report(outputs_dir=outputs_dir)
                final_res_list.extend(res_list)

            elif eval_backend == EvalBackend.OPEN_COMPASS.value:
                eval_config = Summarizer.parse_eval_config(candidate_task)

                work_dir = eval_config.get('work_dir') or 'outputs/default'
                if not os.path.exists(work_dir):
                    raise ValueError(f'work_dir {work_dir} does not exist.')

                res_folder_path = get_latest_folder_path(work_dir=work_dir)
                summary_files = glob.glob(os.path.join(res_folder_path, 'summary', '*.csv'))
                if len(summary_files) == 0:
                    raise ValueError(f'No summary files in {res_folder_path}')

                summary_file_path = summary_files[0]
                # Example: [{'dataset': 'gsm8k', 'version': '1d7fe4', 'metric': 'accuracy', 'mode': 'gen', 'qwen-7b-chat': '53.98'}
                summary_res: List[dict] = csv_to_list(file_path=summary_file_path)
                final_res_list.extend(summary_res)
            elif eval_backend == EvalBackend.VLM_EVAL_KIT.value:
                eval_config = Summarizer.parse_eval_config(candidate_task)

                work_dir = eval_config.get('work_dir') or 'outputs'
                if not os.path.exists(work_dir):
                    raise ValueError(f'work_dir {work_dir} does not exist.')
                
                for model in eval_config['model']:
                    if model['name'] == 'CustomAPIModel':
                        model_name = model['type']
                    else:
                        model_name = model['name']
                    
                    csv_files = glob.glob(os.path.join(work_dir, model_name, '*.csv'))
                    json_files = glob.glob(os.path.join(work_dir, model_name, '*.json'))
                    
                    summary_files = csv_files + json_files
                    for summary_file_path in summary_files:
                        if summary_file_path.endswith('csv'):
                            summary_res: dict = csv_to_list(summary_file_path)[0]
                        elif summary_file_path.endswith('json'):
                            summary_res: dict = json_to_dict(summary_file_path)
                        file_name = os.path.basename(summary_file_path).split('.')[0]
                        final_res_list.append({file_name: summary_res})
                
            elif eval_backend == EvalBackend.THIRD_PARTY.value:
                raise ValueError(f'*** The summarizer for Third party evaluation backend is not supported yet ***')
            else:
                raise ValueError(f'Invalid eval_backend: {eval_backend}')

        return final_res_list

    @staticmethod
    def parse_eval_config(candidate_task):
        eval_config: Union[str, dict] = candidate_task.get('eval_config')
        assert eval_config is not None, 'Please provide eval_config for specific evaluation backend.'

        if isinstance(eval_config, str):
            if eval_config.endswith('.yaml'):
                eval_config: dict = yaml_to_dict(eval_config)
            elif eval_config.endswith('.json'):
                eval_config: dict = json_to_dict(eval_config)
            else:
                raise ValueError(f'Invalid eval_config: {eval_config}')
        return eval_config


if __name__ == '__main__':
    cfg_file = 'registry/tasks/eval_qwen-7b-chat_v100.yaml'
    report_list = Summarizer.get_report_from_cfg(cfg_file)

    print(report_list)



# Copyright (c) Alibaba, Inc. and its affiliates.

import subprocess
import unittest

from evalscope.run import run_task
from evalscope.summarizer import Summarizer
from evalscope.utils import is_module_installed, test_level_list
from evalscope.utils.logger import get_logger

logger = get_logger()


class TestVLMEval(unittest.TestCase):

    def setUp(self) -> None:
        self._check_env('vlmeval')

    def tearDown(self) -> None:
        pass

    @staticmethod
    def _check_env(module_name: str):
        if is_module_installed(module_name):
            logger.info(f'{module_name} is installed.')
        else:
            raise ModuleNotFoundError(f'run: pip install {module_name}')

    @unittest.skipUnless(0 in test_level_list(), 'skip test in current test level')
    def test_run_vlm_eval_local(self):
        task_cfg = {
            'eval_backend': 'VLMEvalKit',
            'eval_config': {
                'data': ['SEEDBench_IMG', 'ChartQA_TEST'],
                'limit': 20,
                'mode': 'all',
                'model': [{
                    'name': 'qwen-vl-chat',
                    'model_path': '../models/Qwen-VL-Chat'
                }],  # model name for VLMEval config
                'nproc': 1,
                'reuse': True,
                'work_dir': 'outputs'
            }
        }

        logger.info(f'>> Start to run task: {task_cfg}')

        run_task(task_cfg)

        logger.info('>> Start to get the report with summarizer ...')
        report_list = Summarizer.get_report_from_cfg(task_cfg)
        logger.info(f'\n>>The report list: {report_list}')

        assert len(report_list) > 0, f'Failed to get report list: {report_list}'


if __name__ == '__main__':
    unittest.main(buffer=False)



import asyncio
import copy
import os
import platform
import sqlite3
import threading
import time
from http import HTTPStatus
from typing import List

import json
import numpy as np
from tqdm import tqdm

from evalscope.perf.arguments import Arguments
from evalscope.perf.http_client import AioHttpClient, test_connection
from evalscope.perf.plugin.registry import ApiRegistry, DatasetRegistry
from evalscope.perf.utils.benchmark_util import BenchmarkData, BenchmarkMetrics
from evalscope.perf.utils.db_util import create_result_table, get_result_db_path, insert_benchmark_data, summary_result
from evalscope.perf.utils.handler import add_signal_handlers, exception_handler
from evalscope.perf.utils.local_server import start_app
from evalscope.utils.logger import get_logger

logger = get_logger()
query_send_completed_event = asyncio.Event()
data_process_completed_event = asyncio.Event()


@exception_handler
async def dispatch_requests_worker(request_queue: asyncio.Queue, args: Arguments):
    query_generator_class = ApiRegistry(args.api)
    query_generator = query_generator_class(args.tokenizer_path)

    def load_prompt(prompt_path_or_text):
        """Load the prompt from a file or directly from the input text."""
        if prompt_path_or_text.startswith('@'):
            with open(prompt_path_or_text[1:], 'r', encoding='utf-8') as file:
                return file.read()
        return prompt_path_or_text

    async def dispatch_request(request):
        """Dispatch a single request with optional rate limiting."""
        await request_queue.put(request)
        if args.rate != -1:
            interval = np.random.exponential(1.0 / args.rate)
            await asyncio.sleep(interval)

    async def dispatch_requests_from_prompt(messages):
        """Generate and dispatch requests based on the given prompt."""
        request = query_generator.build_request(messages, args)
        if args.number is None:
            await dispatch_request(request)
            return 1
        for _ in range(args.number):
            await dispatch_request(request)
        return args.number

    async def dispatch_requests_from_dataset():
        """Generate and dispatch requests based on the dataset."""
        total_query_count = 0
        message_generator_class = DatasetRegistry(args.dataset)
        message_generator = message_generator_class(args)

        for messages in message_generator:
            request = query_generator.build_request(messages, args)
            if request is None:
                continue
            await dispatch_request(request)
            total_query_count += 1
            if args.number and total_query_count >= args.number:
                break

        return total_query_count

    # Load prompt or dataset and dispatch requests accordingly
    if args.prompt:
        prompt = load_prompt(args.prompt)
        messages = [{'role': 'user', 'content': prompt}]
        total_queries = await dispatch_requests_from_prompt(messages)
    elif args.dataset:
        total_queries = await dispatch_requests_from_dataset()
    else:
        raise Exception('Either prompt or dataset is required!')

    return total_queries


@exception_handler
async def send_requests_worker(
    task_id,
    request_queue: asyncio.Queue,
    benchmark_data_queue: asyncio.Queue,
    args: Arguments,
):
    client = AioHttpClient(args)
    async with client:
        while not (query_send_completed_event.is_set() and request_queue.empty()):
            try:
                # Attempt to get a request from the queue with a timeout
                request = await asyncio.wait_for(request_queue.get(), timeout=0.0001)
                request_queue.task_done()
            except asyncio.TimeoutError:
                # If timeout, continue to the next iteration
                continue

            # Initialize benchmark data for the current request
            benchmark_data = BenchmarkData(request=request)
            collected_messages = []
            try:
                # Send the request and process the response
                async for is_error, state_code, response_data in client.post(request):
                    if is_error or state_code != HTTPStatus.OK:
                        logger.error(f'Request: {request} failed, state_code: {state_code}, data: {response_data}')
                        benchmark_data.success = False
                        break
                    if response_data:
                        collected_messages.append(response_data)
                        benchmark_data.chunk_times.append(time.perf_counter())
                        benchmark_data.success = True
                        benchmark_data.update_gpu_usage()
            except Exception as e:
                if response_data:
                    collected_messages.append(response_data)
                benchmark_data.success = False
                logger.exception(e)
                logger.error(f'Request query: {request} exception')
            finally:
                # Record completion time and collected messages
                benchmark_data.completed_time = time.perf_counter()
                benchmark_data.response_messages = collected_messages
                await benchmark_data_queue.put(benchmark_data)


@exception_handler
async def statistic_benchmark_metric_worker(benchmark_data_queue: asyncio.Queue, args: Arguments):
    metrics = BenchmarkMetrics(concurrency=args.parallel)

    api_plugin_class = ApiRegistry(args.api)
    api_plugin = api_plugin_class(args.tokenizer_path)

    result_db_path = get_result_db_path(args.name, args.model)
    # Initialize wandb
    if args.wandb_api_key:
        import wandb
        import datetime
        os.environ['WANDB_SILENT'] = 'true'
        os.environ['WANDB_DIR'] = './outputs'

        wandb.login(key=args.wandb_api_key)
        current_time = datetime.datetime.now().strftime('%Y%m%d_%H%M%S')
        name = args.name if args.name else f'{args.model}_{current_time}'
        wandb.init(project='perf_benchmark', name=name, config=args.to_dict())

    with sqlite3.connect(result_db_path) as con:
        cursor = con.cursor()
        create_result_table(cursor)
        with tqdm(desc='Processing') as pbar:
            while not (data_process_completed_event.is_set() and benchmark_data_queue.empty()):
                try:
                    # Attempt to get benchmark data from the queue with a timeout
                    benchmark_data = await asyncio.wait_for(benchmark_data_queue.get(), timeout=1)
                    benchmark_data_queue.task_done()
                except asyncio.TimeoutError:
                    # If timeout, continue to the next iteration
                    continue

                # Update metrics based on the benchmark data
                metrics.update_metrics(benchmark_data, api_plugin)

                # Insert benchmark data into the database and commit the transaction
                insert_benchmark_data(cursor, benchmark_data)
                con.commit()

                # Create a message with the updated metrics
                message = metrics.create_message()

                # Log the message to wandb if the api key is provided
                if args.wandb_api_key:
                    wandb.log(message)

                # Log the message to the logger every n queries
                if int(metrics.n_total_queries) % args.log_every_n_query == 0:
                    msg = json.dumps(message, ensure_ascii=False, indent=2)
                    logger.info(msg)

                pbar.update(1)  # Update the progress bar

    return metrics, result_db_path


@exception_handler
async def start_server(args: Arguments) -> bool:
    if args.api.startswith('local'):
        #  start local server
        server = threading.Thread(target=start_app, args=(copy.deepcopy(args), ), daemon=True)
        server.start()

        if args.dataset.startswith('speed_benchmark'):
            args.url = 'http://127.0.0.1:8877/v1/completions'
        else:
            args.url = 'http://127.0.0.1:8877/v1/chat/completions'
        args.model = os.path.basename(args.model)

    if not await test_connection(args):
        raise TimeoutError('Test connection failed')


@exception_handler
async def benchmark(args: Arguments) -> None:
    if platform.system() != 'Windows':
        loop = asyncio.get_running_loop()
        add_signal_handlers(loop)

    request_queue = asyncio.Queue()
    benchmark_data_queue = asyncio.Queue()

    async def create_send_request_tasks():
        tasks: List[asyncio.Task] = []
        for idx in range(args.parallel):
            task = asyncio.create_task(send_requests_worker(idx, request_queue, benchmark_data_queue, args))
            tasks.append(task)
        return tasks

    async def run_tasks():
        await start_server(args)

        dispatch_task = asyncio.create_task(dispatch_requests_worker(request_queue, args))
        statistic_benchmark_metric_task = asyncio.create_task(
            statistic_benchmark_metric_worker(benchmark_data_queue, args))
        send_request_tasks = await create_send_request_tasks()

        expected_number_of_queries = await dispatch_task
        await request_queue.join()
        query_send_completed_event.set()

        await asyncio.gather(*send_request_tasks, return_exceptions=True)
        await benchmark_data_queue.join()
        data_process_completed_event.set()

        metrics, result_db_path = await statistic_benchmark_metric_task
        summary_result(args, metrics, expected_number_of_queries, result_db_path)

        await asyncio.sleep(0.250)

    await run_tasks()



# Copyright (c) Alibaba, Inc. and its affiliates.
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# flake8: noqa

import os
import datasets
import pandas as pd


_CITATION = """\
@misc{li2023cmmlu,
      title={CMMLU: Measuring massive multitask language understanding in Chinese},
      author={Haonan Li and Yixuan Zhang and Fajri Koto and Yifei Yang and Hai Zhao and Yeyun Gong and Nan Duan and Timothy Baldwin},
      year={2023},
      eprint={2306.09212},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
"""

_DESCRIPTION = """\
CMMLU is a comprehensive Chinese assessment suite specifically designed to evaluate the advanced knowledge and reasoning abilities of LLMs within the Chinese language and cultural context.
"""

_HOMEPAGE = "https://modelscope.cn/datasets/modelscope/cmmlu/summary"


# _URL = r"https://huggingface.co/datasets/haonan-li/cmmlu/resolve/main/cmmlu_v1_0_1.zip"
_URL = r"https://modelscope.cn/api/v1/datasets/modelscope/cmmlu/repo?Revision=master&FilePath=cmmlu_v1_0_1.zip"

# contains 67 sub-tasks
task_list = [
     'agronomy',
     'anatomy',
     'ancient_chinese',
     'arts',
     'astronomy',
     'business_ethics',
     'chinese_civil_service_exam',
     'chinese_driving_rule',
     'chinese_food_culture',
     'chinese_foreign_policy',
     'chinese_history',
     'chinese_literature',
     'chinese_teacher_qualification',
     'clinical_knowledge',
     'college_actuarial_science',
     'college_education',
     'college_engineering_hydrology',
     'college_law',
     'college_mathematics',
     'college_medical_statistics',
     'college_medicine',
     'computer_science',
     'computer_security',
     'conceptual_physics',
     'construction_project_management',
     'economics',
     'education',
     'electrical_engineering',
     'elementary_chinese',
     'elementary_commonsense',
     'elementary_information_and_technology',
     'elementary_mathematics',
     'ethnology',
     'food_science',
     'genetics',
     'global_facts',
     'high_school_biology',
     'high_school_chemistry',
     'high_school_geography',
     'high_school_mathematics',
     'high_school_physics',
     'high_school_politics',
     'human_sexuality',
     'international_law',
     'journalism',
     'jurisprudence',
     'legal_and_moral_basis',
     'logical',
     'machine_learning',
     'management',
     'marketing',
     'marxist_theory',
     'modern_chinese',
     'nutrition',
     'philosophy',
     'professional_accounting',
     'professional_law',
     'professional_medicine',
     'professional_psychology',
     'public_relations',
     'security_study',
     'sociology',
     'sports_science',
     'traditional_chinese_medicine',
     'virology',
     'world_history',
     'world_religions',
]


class CMMLUConfig(datasets.BuilderConfig):
    def __init__(self, **kwargs):
        super().__init__(version=datasets.Version("1.0.1"), **kwargs)
        # V1.0.1 Fix: One comma missing in word_religions.csv
        # V1.0.0 Init version


class CMMLU(datasets.GeneratorBasedBuilder):
    BUILDER_CONFIGS = [
        CMMLUConfig(name=task_name) for task_name in task_list
    ]

    def _info(self):
        features = datasets.Features(
            {
                "Question": datasets.Value("string"),
                "A": datasets.Value("string"),
                "B": datasets.Value("string"),
                "C": datasets.Value("string"),
                "D": datasets.Value("string"),
                "Answer": datasets.Value("string"),
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        data_dir = dl_manager.download_and_extract(_URL)
        task_name = self.config.name
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, f"test/{task_name}.csv"),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split("dev"),
                gen_kwargs={
                    "filepath": os.path.join(data_dir, f"dev/{task_name}.csv"),
                },
            ),
        ]

    def _generate_examples(self, filepath):
        df = pd.read_csv(filepath, header=0, index_col=0, encoding="utf-8")
        for i, instance in enumerate(df.to_dict(orient="records")):
            yield i, instance


# Copyright (c) Alibaba, Inc. and its affiliates.
import os
from typing import Union

from evalscope.third_party.longbench_write.infer import run_infer
from evalscope.third_party.longbench_write.eval import run_eval
from evalscope.utils import yaml_to_dict, json_to_dict
from evalscope.utils import get_logger

logger = get_logger()

"""
Entry file for LongWriter evaluation.
"""


def run_task(task_cfg: Union[str, dict]):

    if isinstance(task_cfg, str):
        if task_cfg.endswith('.yaml'):
            task_cfg: dict = yaml_to_dict(task_cfg)
        elif task_cfg.endswith('.json'):
            task_cfg: dict = json_to_dict(task_cfg)
        else:
            raise ValueError(f'Unsupported file format: {task_cfg}, should be yaml or json file.')

    # Parse task configuration
    stage: list = task_cfg.get('stage', ['infer', 'eval_l', 'eval_q'])
    model: str = task_cfg.get('model')
    input_data_path: str = task_cfg.get('input_data_path')
    output_dir: str = task_cfg.get('output_dir')

    infer_config: dict = task_cfg.get('infer_config')
    eval_config: dict = task_cfg.get('eval_config')
    assert infer_config is not None and eval_config is not None, 'Please provide infer_config and eval_config.'

    # Run inference process
    pred_res_path = run_infer(model=model,
                              data_path=input_data_path or os.path.join(os.path.dirname(__file__), 'resources/longbench_write.jsonl'),
                              output_dir=output_dir,
                              api_config=dict(
                                  openai_api_key=infer_config.get('openai_api_key'),
                                  openai_api_base=infer_config.get('openai_api_base'),
                                  is_chat=infer_config.get('is_chat', True),
                                  verbose=infer_config.get('verbose', False),
                              ),
                              generation_kwargs=infer_config.get('generation_kwargs'),
                              enable='infer' in stage)

    # Run eval process
    run_eval(model=model,
             pred_path=pred_res_path,
             output_dir=output_dir,
             prompt_template_path=os.path.join(os.path.dirname(__file__), 'resources/judge.txt'),
             openai_api_key=eval_config.get('openai_api_key'),
             openai_api_base=eval_config.get('openai_api_base'),
             openai_gpt_model=eval_config.get('openai_gpt_model'),
             generation_kwargs=eval_config.get('generation_kwargs'),
             proc_num=eval_config.get('proc_num', 16),
             stage=stage)


if __name__ == '__main__':
    # Note: evaluation task configuration can also be loaded from yaml or json file.
    # task_cfg = os.path.join(os.path.dirname(__file__), 'default_task.yaml')
    # task_cfg = os.path.join(os.path.dirname(__file__), 'default_task.json')
    task_cfg = dict(stage=['infer', 'eval_l', 'eval_q'],
                    model='ZhipuAI/LongWriter-glm4-9b',  # or /path/to/your_model_dir
                    input_data_path=None,
                    output_dir='./outputs',

                    infer_config={
                        'openai_api_base': 'http://127.0.0.1:8000/v1/chat/completions',
                        'is_chat': True,
                        'verbose': False,
                        'generation_kwargs': {'max_new_tokens': 32768, 'temperature': 0.5, 'repetition_penalty': 1.0},
                    },

                    eval_config={
                        'openai_api_key': 'YOUR_OPENAI_API_KEY',
                        'openai_api_base': 'https://api.openai.com/v1/chat/completions',
                        'openai_gpt_model': 'gpt-4o-2024-05-13',
                        'generation_kwargs': {'max_new_tokens': 1024, 'temperature': 0.5, 'stop': None},
                        'proc_num': 16,
                    },
                    )

    run_task(task_cfg=task_cfg)



import asyncio
import platform
from argparse import Namespace

from evalscope.perf.arguments import Arguments, parse_args
from evalscope.perf.benchmark import benchmark
from evalscope.perf.utils.handler import add_signal_handlers
from evalscope.utils.logger import get_logger
from evalscope.utils.utils import seed_everything

logger = get_logger()


def run_perf_benchmark(args):
    if isinstance(args, dict):
        args = Arguments(**args)
    elif isinstance(args, Namespace):
        args = Arguments.from_args(args)
    seed_everything(args.seed)

    logger.info('Starting benchmark...')
    logger.info(args)

    if platform.system() == 'Windows':
        asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())

    loop = asyncio.get_event_loop()
    if platform.system() != 'Windows':
        add_signal_handlers(loop)
    loop.run_until_complete(benchmark(args))


if __name__ == '__main__':
    args = Arguments.from_args(parse_args())
    run_perf_benchmark(args)



import os
from typing import Any, Dict, Iterator, List, Mapping, Optional
from modelscope.utils.hf_util import GenerationConfig
from langchain_core.callbacks.manager import CallbackManagerForLLMRun
from langchain_core.language_models.llms import LLM as BaseLLM
from evalscope.models.model_adapter import ChatGenerationModelAdapter
from langchain_openai import ChatOpenAI


class LLM:
    @staticmethod
    def load(**kw):
        api_base = kw.get('api_base', None)
        if api_base:
            return ChatOpenAI(
                model_name=kw.get('model_name', ''),
                openai_api_base=api_base,
                openai_api_key=kw.get('api_key', 'EMPTY'),
            )
        else:
            return LocalLLM(**kw)


class LocalLLM(BaseLLM):
    """A custom LLM that loads a model from a given path and performs inference."""

    model_name_or_path: str
    model_revision: str = 'master'
    template_type: str = 'default'
    model_name: Optional[str]
    model: Optional[ChatGenerationModelAdapter]
    generation_config: Optional[Dict]

    def __init__(self, **kw):
        super().__init__(**kw)
        self.model_name = os.path.basename(self.model_name_or_path)
        self.model = ChatGenerationModelAdapter(
            model_id=self.model_name_or_path,
            model_revision=self.model_revision,
            template_type=self.template_type,
            generation_config=GenerationConfig(**self.generation_config) if self.generation_config else None,
        )

    def _call(
        self,
        prompt: str,
        stop: Optional[List[str]] = None,
        run_manager: Optional[CallbackManagerForLLMRun] = None,
        **kwargs: Any,
    ) -> str:
        """Run the LLM on the given input."""
        infer_cfg = {'stop': stop}

        response = self.model._model_generate(prompt, infer_cfg)
        return response

    @property
    def _identifying_params(self) -> Dict[str, Any]:
        """Return a dictionary of identifying parameters."""
        return {
            # The model name allows users to specify custom token counting
            # rules in LLM monitoring applications (e.g., in LangSmith users
            # can provide per token pricing for their model and monitor
            # costs for the given LLM.)
            'model_name': self.model_name,
            'revision': self.model_revision,
        }

    @property
    def _llm_type(self) -> str:
        """Get the type of language model used by this chat model. Used for logging purposes only."""
        return self.model_name



from typing import Optional
from mteb import AbsTaskRetrieval
from mteb import HFDataLoader as CustomDataLoader
from mteb.abstasks.TaskMetadata import TaskMetadata


class CustomRetrieval(AbsTaskRetrieval):
    metadata: TaskMetadata
    ignore_identical_ids: bool = True

    def __init__(
        self, dataset_path: Optional[str] = "custom_eval/text/retrieval", **kwargs
    ):
        super().__init__(**kwargs)
        self.metadata = TaskMetadata(
            name="CustomRetrieval",
            description="CustomRetrieval Task",
            reference=None,
            dataset={
                "path": dataset_path,
                "revision": "v1",
            },
            type="Retrieval",
            category="s2p",
            modalities=["text"],
            eval_splits=["test"],
            eval_langs=["cmn-Hans"],
            main_score="recall_at_5",
            date=None,
            domains=None,
            task_subtypes=None,
            license=None,
            annotations_creators=None,
            dialect=None,
            sample_creation=None,
            bibtex_citation="",
            descriptive_stats={},
        )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return
        self.corpus, self.queries, self.relevant_docs = {}, {}, {}
        dataset_path = self.metadata_dict["dataset"]["path"]

        for split in kwargs.get("eval_splits", self.metadata_dict["eval_splits"]):
            corpus, queries, qrels = CustomDataLoader(
                data_folder=dataset_path,
                streaming=False,
                keep_in_memory=False,
            ).load(split=split)
            # Conversion from DataSet
            queries = {query["id"]: query["text"] for query in queries}
            corpus = {doc["id"]: {"text": doc["text"]} for doc in corpus}
            self.corpus[split], self.queries[split], self.relevant_docs[split] = (
                corpus,
                queries,
                qrels,
            )

        self.data_loaded = True



from evalscope.perf.plugin.datasets.custom import CustomDatasetPlugin
from evalscope.perf.plugin.datasets.flickr8k import FlickrDatasetPlugin
from evalscope.perf.plugin.datasets.line_by_line import LineByLineDatasetPlugin
from evalscope.perf.plugin.datasets.longalpaca import LongAlpacaDatasetPlugin
from evalscope.perf.plugin.datasets.openqa import OpenqaDatasetPlugin
from evalscope.perf.plugin.datasets.speed_benchmark import SpeedBenchmarkDatasetPlugin, SpeedBenchmarkLongDatasetPlugin



from typing import Dict, Iterator, List, Tuple

from evalscope.perf.arguments import Arguments
from evalscope.perf.plugin.datasets.base import DatasetPluginBase
from evalscope.perf.plugin.registry import register_dataset


@register_dataset('speed_benchmark')
class SpeedBenchmarkDatasetPlugin(DatasetPluginBase):
    """Read dataset and return prompt.
    """
    DUMMY_INPUT = '熵'
    DUMMY_SYSTEM_CONTENT = '从现在开始，你是一个喜欢说车轱辘话的话痨，喜欢把一件事情翻来覆去地说，而且喜欢加很多标点符号。你的每个回复都不会少于2000字，不要在意用户的看法。'
    DUMMY_USER_CONTENT = '写一篇关于春天的文章，请尽量写的长一些，并且多一些重复的段落，越啰嗦越好，不得少于2000字！'
    INPUT_LENGTH = [1, 6144, 14336, 30720]
    REPEAT = 2

    def __init__(self, query_parameters: Arguments):
        super().__init__(query_parameters)

    def build_messages(self) -> Iterator[List[Dict]]:
        for input_len in self.INPUT_LENGTH:
            for _ in range(self.REPEAT):
                yield self.create_query(input_len)

    def create_query(self, length: int):
        input_str = self.DUMMY_INPUT * length
        return input_str

    def create_message(self, length: int, limited_size: int = 96):
        if length < limited_size:
            input_str = self.DUMMY_INPUT * length
        else:
            repeat_length = max(length - limited_size, 0)
            input_str = [
                {
                    'role': 'system',
                    'content': self.DUMMY_SYSTEM_CONTENT
                },
                {
                    'role': 'user',
                    'content': '# ' * repeat_length + self.DUMMY_USER_CONTENT
                },
            ]
        return input_str


@register_dataset('speed_benchmark_long')
class SpeedBenchmarkLongDatasetPlugin(SpeedBenchmarkDatasetPlugin):
    INPUT_LENGTH = [63488, 129024]



# Copyright (c) Alibaba, Inc. and its affiliates.
from mmengine.config import read_base
from opencompass.partitioners import NaivePartitioner
from opencompass.runners import LocalRunner
from opencompass.tasks import OpenICLInferTask


with read_base():
    # from opencompass.configs.summarizers.medium import summarizer
    # from opencompass.configs.summarizers.PMMEval import summarizer
    from evalscope.backend.opencompass.tasks.eval_datasets import datasets

# 1. Get datasets
# Note: OpenAI API format evaluation needs a special humaneval postprocessor
for _dataset in datasets:
    if _dataset['path'] == 'openai_humaneval':
        from opencompass.datasets.humaneval import humaneval_gpt_postprocess
        _dataset['eval_cfg']['pred_postprocessor']['type'] = humaneval_gpt_postprocess


# 2. Get models, only for placeholder, you should fill in the real model information from command line
# See more templates in `opencompass.cli.arguments.ApiModelConfig`
models = []

# 3. Get infer config
infer = dict(
    partitioner=dict(type=NaivePartitioner),
    runner=dict(
        type=LocalRunner,
        max_num_workers=4,
        task=dict(type=OpenICLInferTask)),
)
